import pytest

from mollie.api.error import IdentifierError
from mollie.api.objects.capture import Capture
from mollie.api.objects.chargeback import Chargeback
from mollie.api.objects.customer import Customer
from mollie.api.objects.mandate import Mandate
from mollie.api.objects.method import Method
from mollie.api.objects.order import Order
from mollie.api.objects.payment import Payment
from mollie.api.objects.refund import Refund
from mollie.api.objects.settlement import Settlement
from mollie.api.objects.subscription import Subscription

from .utils import assert_list_object

PAYMENT_ID = 'tr_7UhSN1zuXS'
REFUND_ID = 're_4qqhO89gsT'
CHARGEBACK_ID = 'chb_n9z0tp'
CUSTOMER_ID = 'cst_8wmqcHMN4U'
SETTLEMENT_ID = 'stl_jDk30akdN'
MANDATE_ID = 'mdt_h3gAaD5zP'
SUBSCRIPTION_ID = 'sub_rVKGtNd6s3'
ORDER_ID = 'ord_kEn1PlbGa'


def test_list_payments(client, response):
    """Retrieve a list of payments."""
    response.get('https://api.mollie.com/v2/payments', 'payments_list')

    payments = client.payments.list()
    assert_list_object(payments, Payment)


def test_create_payment(client, response):
    """Create a new payment."""
    response.post('https://api.mollie.com/v2/payments', 'payment_single')

    payment = client.payments.create(
        {
            'amount': {'currency': 'EUR', 'value': '10.00'},
            'description': 'Order #12345',
            'redirectUrl': 'https://webshop.example.org/order/12345/',
            'webhookUrl': 'https://webshop.example.org/payments/webhook/',
            'method': 'ideal',
        })
    assert payment.id == PAYMENT_ID


def test_cancel_payment(client, response):
    """Cancel existing payment."""
    response.delete('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_canceled', 200)

    canceled_payment = client.payments.delete(PAYMENT_ID)
    assert isinstance(canceled_payment, Payment)
    assert canceled_payment.is_canceled() is True
    assert canceled_payment.canceled_at == '2018-03-20T09:28:37+00:00'
    assert canceled_payment.id == PAYMENT_ID


def test_cancel_payment_invalid_id(client):
    """Verify that an invalid payment id is validated and an error is raised."""
    with pytest.raises(IdentifierError):
        client.payments.delete('invalid')


def test_get_single_payment(client, response):
    """Retrieve a single payment by payment id."""
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')
    response.get('https://api.mollie.com/v2/payments/%s/refunds' % PAYMENT_ID, 'refunds_list')
    response.get('https://api.mollie.com/v2/payments/%s/chargebacks' % PAYMENT_ID,
                 'chargebacks_list')
    response.get('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_single')
    response.get('https://api.mollie.com/v2/customers/{cust}/mandates/{man}'.format(
        cust=CUSTOMER_ID, man=MANDATE_ID), 'customer_mandate_single')
    response.get('https://api.mollie.com/v2/customers/{cust}/subscriptions/{sub}'.format(
        cust=CUSTOMER_ID, sub=SUBSCRIPTION_ID), 'subscription_single')
    response.get('https://api.mollie.com/v2/orders/{order_id}'.format(order_id=ORDER_ID),
                 'order_single')

    payment = client.payments.get(PAYMENT_ID)
    assert isinstance(payment, Payment)
    # properties
    assert payment.resource == 'payment'
    assert payment.id == PAYMENT_ID
    assert payment.mode == 'test'
    assert payment.created_at == '2018-03-20T09:13:37+00:00'
    assert payment.status == Payment.STATUS_OPEN
    assert payment.is_cancelable is False
    assert payment.paid_at is None
    assert payment.canceled_at is None
    assert payment.authorized_at is None
    assert payment.expires_at == '2018-03-20T09:28:37+00:00'
    assert payment.expired_at is None
    assert payment.failed_at is None
    assert payment.amount == {'value': '10.00', 'currency': 'EUR'}
    assert payment.amount_refunded is None
    assert payment.amount_remaining is None
    assert payment.description == 'Order #12345'
    assert payment.redirect_url == 'https://webshop.example.org/order/12345/'
    assert payment.webhook_url == 'https://webshop.example.org/payments/webhook/'
    assert payment.method == Method.IDEAL
    assert payment.metadata == {'order_id': '12345'}
    assert payment.locale is None
    assert payment.country_code is None
    assert payment.profile_id == 'pfl_QkEhN94Ba'
    assert payment.settlement_amount == {
        "currency": "EUR",
        "value": "39.75"
    }
    assert payment.settlement_id == 'stl_jDk30akdN'
    assert payment.customer_id == CUSTOMER_ID
    assert payment.sequence_type == Payment.SEQUENCETYPE_RECURRING
    assert payment.mandate_id == MANDATE_ID
    assert payment.subscription_id == SUBSCRIPTION_ID
    assert payment.order_id == ORDER_ID
    assert payment.application_fee is None
    assert payment.details is None
    # properties from _links
    assert payment.checkout_url == 'https://www.mollie.com/payscreen/select-method/7UhSN1zuXS'
    assert payment.refunds is not None
    assert payment.chargebacks is not None
    assert payment.mandate is not None
    assert payment.subscription is not None
    assert payment.customer is not None
    assert isinstance(payment.order, Order)
    # additional methods
    assert payment.is_open() is True
    assert payment.is_pending() is False
    assert payment.is_canceled() is False
    assert payment.is_expired() is False
    assert payment.is_paid() is False
    assert payment.is_failed() is False
    assert payment.is_authorized() is False
    assert payment.has_refunds() is True
    assert payment.can_be_refunded() is False
    assert payment.has_sequence_type_first() is False
    assert payment.has_sequence_type_recurring() is True


def test_payment_get_related_refunds(client, response):
    """Retrieve a list of refunds related to a payment."""
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')
    response.get('https://api.mollie.com/v2/payments/%s/refunds' % PAYMENT_ID, 'refunds_list')

    payment = client.payments.get(PAYMENT_ID)
    refunds = payment.refunds
    assert_list_object(refunds, Refund)


def test_payment_get_related_chargebacks(client, response):
    """Get chargebacks related to payment id."""
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')
    response.get('https://api.mollie.com/v2/payments/%s/chargebacks' % PAYMENT_ID,
                 'chargebacks_list')

    payment = client.payments.get(PAYMENT_ID)
    chargebacks = payment.chargebacks
    assert_list_object(chargebacks, Chargeback)


def test_payment_get_related_captures(client, response):
    """Get captures related to payment id."""
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')
    response.get('https://api.mollie.com/v2/payments/%s/captures' % PAYMENT_ID, 'captures_list')

    payment = client.payments.get(PAYMENT_ID)
    captures = payment.captures
    assert_list_object(captures, Capture)


def test_payment_get_related_settlement(client, response):
    """Get the settlement related to the payment."""
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')
    response.get('https://api.mollie.com/v2/settlements/%s' % SETTLEMENT_ID, 'settlement_single')

    payment = client.payments.get(PAYMENT_ID)
    settlement = payment.settlement
    assert isinstance(settlement, Settlement)
    assert settlement.id == SETTLEMENT_ID


def test_payment_get_related_mandate(client, response):
    """Get the mandate related to the payment."""
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')
    response.get('https://api.mollie.com/v2/customers/{cust}/mandates/{man}'.format(
        cust=CUSTOMER_ID, man=MANDATE_ID), 'customer_mandate_single')

    payment = client.payments.get(PAYMENT_ID)
    mandate = payment.mandate
    assert isinstance(mandate, Mandate)
    assert mandate.id == MANDATE_ID


def test_payment_get_related_subscription(client, response):
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')
    response.get('https://api.mollie.com/v2/customers/{cust}/subscriptions/{sub}'.format(
        cust=CUSTOMER_ID, sub=SUBSCRIPTION_ID), 'subscription_single')

    payment = client.payments.get(PAYMENT_ID)
    subscription = payment.subscription
    assert isinstance(subscription, Subscription)
    assert subscription.id == SUBSCRIPTION_ID


def test_payment_get_related_customer(client, response):
    """Get customer related to payment."""
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')
    response.get('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_single')

    payment = client.payments.get(PAYMENT_ID)
    customer = payment.customer
    assert isinstance(customer, Customer)
    assert customer.id == CUSTOMER_ID


def test_update_payment(client, response):
    """Update an existing payment."""
    response.patch('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_updated')

    data = {
        'description': 'Order #12346',
        'redirectUrl': 'https://webshop.example.org/order/12346/',
        'webhookUrl': 'https://webshop.example.org/payments/webhook/',
        'metadata': {"order_id": "12346"},
    }
    updated_payment = client.payments.update(PAYMENT_ID, data)
    assert isinstance(updated_payment, Payment)
    assert updated_payment.description == 'Order #12346'



from .base import Base
from .issuer import Issuer
from .list import List


class Method(Base):
    @classmethod
    def get_resource_class(cls, client):
        from ..resources.methods import Methods
        return Methods(client)

    BANCONTACT = 'bancontact'
    BANKTRANSFER = 'banktransfer'
    BELFIUS = 'belfius'
    CREDITCARD = 'creditcard'
    DIRECTDEBIT = 'directdebit'
    EPS = 'eps'
    GIFTCARD = 'giftcard'
    GIROPAY = 'giropay'
    IDEAL = 'ideal'
    INGHOMEPAY = 'inghomepay'
    KBC = 'kbc'
    KLARNAPAYLATER = 'klarnapaylater'
    KLARNASLICEIT = 'klarnasliceit'
    MEALVOUCHER = 'mealvoucher'
    MISTERCASH = 'mistercash'
    MYBANK = 'mybank'
    PAYPAL = 'paypal'
    PAYSAFECARD = 'paysafecard'
    PODIUMCADEAUKAART = 'podiumcadeaukaart'
    PRZELEWY24 = 'przelewy24'
    SOFORT = 'sofort'

    @property
    def description(self):
        return self._get_property('description')

    @property
    def id(self):
        return self._get_property('id')

    @property
    def minimum_amount(self):
        return self._get_property('minimumAmount')

    @property
    def maximum_amount(self):
        return self._get_property('maximumAmount')

    @property
    def pricing(self):
        return self._get_property('pricing')

    @property
    def image_svg(self):
        try:
            images = self._get_property('image')
            return images['svg']
        except KeyError:
            return None

    @property
    def image_size1x(self):
        try:
            images = self._get_property('image')
            return images['size1x']
        except KeyError:
            return None

    @property
    def image_size2x(self):
        try:
            images = self._get_property('image')
            return images['size2x']
        except KeyError:
            return None

    @property
    def issuers(self):
        """Return the list of available issuers for this payment method."""
        issuers = self._get_property('issuers') or []
        result = {
            '_embedded': {
                'issuers': issuers,
            },
            'count': len(issuers),
        }
        return List(result, Issuer)



import pytest

from mollie.api.error import IdentifierError
from mollie.api.objects.profile import Profile
from mollie.api.resources.profiles import Profiles

from .utils import assert_list_object

PROFILE_ID = 'pfl_v9hTwCvYqw'


def test_profile_resource_class(oauth_client, response):
    response.get('https://api.mollie.com/v2/profiles/%s' % PROFILE_ID, 'profile_single')
    oauth_client.profiles.get(PROFILE_ID)

    assert isinstance(Profile.get_resource_class(oauth_client), Profiles)


def test_profiles_get_raises_identifier_error(oauth_client, response):
    response.get('https://api.mollie.com/v2/profiles/%s' % PROFILE_ID, 'profile_single')
    oauth_client.profiles.get(PROFILE_ID)

    with pytest.raises(IdentifierError):
        Profiles(oauth_client).get(None)


def test_create_profile(oauth_client, response):
    """Create a new profile."""
    response.post('https://api.mollie.com/v2/profiles', 'profile_new')

    profile = oauth_client.profiles.create({
      'name': 'My website name',
      'website': 'https://www.mywebsite.com',
      'email': 'info@mywebsite.com',
      'phone': '+31208202070',
      'categoryCode': '5399',
      'mode': 'live',
    })
    assert isinstance(profile, Profile)
    assert profile.id == PROFILE_ID


def test_update_profile(oauth_client, response):
    """Update an existing profile."""
    response.patch('https://api.mollie.com/v2/profiles/%s' % PROFILE_ID, 'profile_updated')

    updated_profile = oauth_client.profiles.update(PROFILE_ID, {
        'name': 'My website name updated',
        'email': 'updated-profile@example.org',
    })
    assert isinstance(updated_profile, Profile)
    assert updated_profile.name == 'My website name updated'
    assert updated_profile.email == 'updated-profile@example.org'


def test_delete_profile(oauth_client, response):
    """Delete a profile."""
    response.delete('https://api.mollie.com/v2/profiles/%s' % PROFILE_ID, 'empty')

    deleted_profile = oauth_client.profiles.delete('pfl_v9hTwCvYqw')
    assert deleted_profile == {}


def test_list_profiles(oauth_client, response):
    """Retrieve a list of existing profiles."""
    response.get('https://api.mollie.com/v2/profiles', 'profiles_list')

    profiles = oauth_client.profiles.list()
    assert_list_object(profiles, Profile)


def test_get_profile(oauth_client, response):
    """Retrieve a single profile."""
    response.get('https://api.mollie.com/v2/profiles/%s' % PROFILE_ID, 'profile_single')
    response.get('https://api.mollie.com/v2/chargebacks?profileId=%s' % PROFILE_ID, 'chargebacks_list')
    response.get('https://api.mollie.com/v2/methods?profileId=%s' % PROFILE_ID, 'methods_list')
    response.get('https://api.mollie.com/v2/payments?profileId=%s' % PROFILE_ID, 'payments_list')
    response.get('https://api.mollie.com/v2/refunds?profileId=%s' % PROFILE_ID, 'refunds_list')

    profile = oauth_client.profiles.get(PROFILE_ID)
    chargebacks = oauth_client.profile_chargebacks.with_parent_id(PROFILE_ID).list()
    methods = oauth_client.profile_methods.with_parent_id(PROFILE_ID).list()
    payments = oauth_client.profile_payments.with_parent_id(PROFILE_ID).list()
    refunds = oauth_client.profile_refunds.with_parent_id(PROFILE_ID).list()

    assert isinstance(profile, Profile)
    assert profile.id == PROFILE_ID
    assert profile.name == 'My website name'
    assert profile.email == 'info@mywebsite.com'
    assert profile.mode == 'live'
    assert profile.resource == 'profile'
    assert profile.created_at == '2018-03-20T09:28:37+00:00'
    assert profile.website == 'https://www.mywebsite.com'
    assert profile.phone == '+31208202070'
    assert profile.category_code == 5399
    assert profile.status == 'verified'
    assert profile.review == {"status": "pending"}
    assert profile.checkout_preview_url == 'https://www.mollie.com/payscreen/preview/pfl_v9hTwCvYqw'
    assert profile.chargebacks == chargebacks
    assert profile.methods == methods
    assert profile.payments == payments
    assert profile.refunds == refunds
    assert profile.is_unverified() is False
    assert profile.is_verified() is True
    assert profile.is_blocked() is False


def test_profile_enable_payment_method(oauth_client, response):
    response.get('https://api.mollie.com/v2/profiles/%s' % PROFILE_ID, 'profile_new')
    response.post(
        'https://api.mollie.com/v2/profiles/%s/methods/%s' % (PROFILE_ID, 'bancontact'),
        'profile_enable_payment_method'
    )

    profile = oauth_client.profiles.get(PROFILE_ID)
    method = oauth_client.profile_methods.on(profile, 'bancontact').create()
    assert method.id == 'bancontact'


def test_profile_disable_payment_method(oauth_client, response):
    response.get('https://api.mollie.com/v2/profiles/%s' % PROFILE_ID, 'profile_new')
    response.delete('https://api.mollie.com/v2/profiles/%s/methods/%s' % (PROFILE_ID, 'bancontact'), 'empty', 204)

    profile = oauth_client.profiles.get(PROFILE_ID)
    method = oauth_client.profile_methods.on(profile, 'bancontact').delete()
    assert method == {}



from mollie.api.objects.list import List


def assert_list_object(obj, object_type):
    """Assert that a List object is correctly working, and has sane contents."""
    assert isinstance(obj, List), 'Object {obj} is not a List instance.'.format(obj=obj)
    assert isinstance(obj.count, int), 'List count is not an integer.'
    assert obj.count > 0

    # verify items in list
    items = []
    for item in obj:
        assert isinstance(item, object_type)
        assert item.id is not None
        items.append(item.id)

    assert len(items) == obj.count, "Items in list don't match list count."
    assert len(set(items)) == obj.count, 'Not all object ids in the list are unique.'



from .base import Base


class Payment(Base):
    @classmethod
    def get_resource_class(cls, client):
        from ..resources.payments import Payments
        return Payments(client)

    STATUS_OPEN = 'open'
    STATUS_PENDING = 'pending'
    STATUS_CANCELED = 'canceled'
    STATUS_EXPIRED = 'expired'
    STATUS_FAILED = 'failed'
    STATUS_PAID = 'paid'
    STATUS_AUTHORIZED = 'authorized'

    SEQUENCETYPE_ONEOFF = 'oneoff'
    SEQUENCETYPE_FIRST = 'first'
    SEQUENCETYPE_RECURRING = 'recurring'

    # Documented properties

    @property
    def resource(self):
        return self._get_property('resource')

    @property
    def id(self):
        return self._get_property('id')

    @property
    def mode(self):
        return self._get_property('mode')

    @property
    def created_at(self):
        return self._get_property('createdAt')

    @property
    def status(self):
        return self._get_property('status')

    @property
    def is_cancelable(self):
        return self._get_property('isCancelable')

    @property
    def authorized_at(self):
        return self._get_property('authorizedAt')

    @property
    def paid_at(self):
        return self._get_property('paidAt')

    @property
    def canceled_at(self):
        return self._get_property('canceledAt')

    @property
    def expires_at(self):
        return self._get_property('expiresAt')

    @property
    def expired_at(self):
        return self._get_property('expiredAt')

    @property
    def failed_at(self):
        return self._get_property('failedAt')

    @property
    def amount(self):
        return self._get_property('amount')

    @property
    def amount_refunded(self):
        return self._get_property('amountRefunded')

    @property
    def amount_remaining(self):
        return self._get_property('amountRemaining')

    @property
    def description(self):
        return self._get_property('description')

    @property
    def redirect_url(self):
        return self._get_property('redirectUrl')

    @property
    def webhook_url(self):
        return self._get_property('webhookUrl')

    @property
    def method(self):
        return self._get_property('method')

    @property
    def metadata(self):
        return self._get_property('metadata')

    @property
    def locale(self):
        return self._get_property('locale')

    @property
    def country_code(self):
        return self._get_property('countryCode')

    @property
    def profile_id(self):
        return self._get_property('profileId')

    @property
    def settlement_amount(self):
        return self._get_property('settlementAmount')

    @property
    def settlement_id(self):
        return self._get_property('settlementId')

    @property
    def customer_id(self):
        return self._get_property('customerId')

    @property
    def sequence_type(self):
        return self._get_property('sequenceType')

    @property
    def mandate_id(self):
        return self._get_property('mandateId')

    @property
    def subscription_id(self):
        return self._get_property('subscriptionId')

    @property
    def order_id(self):
        return self._get_property('orderId')

    @property
    def application_fee(self):
        return self._get_property('applicationFee')

    @property
    def details(self):
        return self._get_property('details')

    # documented _links

    @property
    def checkout_url(self):
        return self._get_link('checkout')

    @property
    def refunds(self):
        """Return the refunds related to this payment."""
        return self.client.payment_refunds.on(self).list()

    @property
    def chargebacks(self):
        """Return the chargebacks related to this payment."""
        return self.client.payment_chargebacks.on(self).list()

    @property
    def captures(self):
        """Return the captures related to this payment"""
        return self.client.captures.on(self).list()

    @property
    def settlement(self):
        """
        Return the settlement for this payment.
        """
        return self.client.settlements.get(self.settlement_id)

    @property
    def mandate(self):
        """Return the mandate for this payment."""
        return self.client.customer_mandates.with_parent_id(self.customer_id).get(self.mandate_id)

    @property
    def subscription(self):
        """Return the subscription for this payment."""
        return self.client.customer_subscriptions.with_parent_id(self.customer_id).get(self.subscription_id)

    @property
    def customer(self):
        """Return the customer for this payment."""
        return self.client.customers.get(self.customer_id)

    @property
    def order(self):
        """Return the order for this payment. """
        from ..resources.orders import Order
        url = self._get_link('order')
        if url:
            resp = self.client.orders.perform_api_call(self.client.orders.REST_READ, url)
            return Order(resp, self.client)

    # additional methods

    def is_open(self):
        return self._get_property('status') == self.STATUS_OPEN

    def is_pending(self):
        return self._get_property('status') == self.STATUS_PENDING

    def is_canceled(self):
        return self._get_property('status') == self.STATUS_CANCELED

    def is_expired(self):
        return self._get_property('status') == self.STATUS_EXPIRED

    def is_failed(self):
        return self._get_property('status') == self.STATUS_FAILED

    def is_authorized(self):
        return self._get_property('status') == self.STATUS_AUTHORIZED

    def is_paid(self):
        return self._get_property('paidAt') is not None

    def has_refunds(self):
        return self._get_link('refunds') is not None

    def can_be_refunded(self):
        return self._get_property('amountRemaining') is not None

    def has_sequence_type_first(self):
        return self._get_property('sequenceType') == self.SEQUENCETYPE_FIRST

    def has_sequence_type_recurring(self):
        return self._get_property('sequenceType') == self.SEQUENCETYPE_RECURRING



from .base import Base


class Organization(Base):
    @classmethod
    def get_resource_class(cls, client):
        from ..resources.organizations import Organizations
        return Organizations(client)

    @property
    def id(self):
        return self._get_property('id')

    @property
    def name(self):
        return self._get_property('name')

    @property
    def email(self):
        return self._get_property('email')

    @property
    def locale(self):
        return self._get_property('locale')

    @property
    def address(self):
        return self._get_property('address')

    @property
    def registration_number(self):
        return self._get_property('registrationNumber')

    @property
    def vat_number(self):
        return self._get_property('vatNumber')

    @property
    def dashboard(self):
        return self._get_link('dashboard')



from mollie.api.objects.payment import Payment

from .utils import assert_list_object

SETTLEMENT_ID = 'stl_jDk30akdN'


def test_list_customer_payments(oauth_client, response):
    """Retrieve a list of payments related to a settlement id."""
    response.get('https://api.mollie.com/v2/settlements/%s/payments' % SETTLEMENT_ID, 'settlement_payments_multiple')

    payments = oauth_client.settlement_payments.with_parent_id(SETTLEMENT_ID).list()
    assert_list_object(payments, Payment)



import logging
import os
import time

import pytest
import responses

from mollie.api.client import Client

logger = logging.getLogger('mollie.pytest.fixtures')


@pytest.fixture(scope='session')
def client():
    """Setup a Mollie API client object."""
    api_key = os.environ.get('MOLLIE_API_KEY', 'test_test')

    client = Client()
    client.set_api_key(api_key)
    return client


@pytest.fixture(scope='session')
def oauth_token():
    """Return a valid oauth token for resuming an existing OAuth client."""
    token = {
        "access_token": "access_H35x4awgfxPKPuHRjpKMAkP2bOgUs",
        "expires_in": 3600,
        "token_type": "bearer",
        "scope": [
            "profiles.read",
        ],
        "refresh_token": "refresh_asfqqJCxj9TjU8B544r44Tsu9bOgUs",
        "expires_at": time.time() + 300  # token is valid for another 5 minutes
    }
    return token


@pytest.fixture(scope='session')
def oauth_client(oauth_token):
    """Setup a Mollie API client with initialized OAuth2 authentication."""
    client_id = "app_nvQQ4mGHqprcfFFqpnmbOgUs"
    client_secret = "2Tuc4qk8U6kCA8qBV3Fb2wwceDDfeRebDQpbOgUs"
    redirect_uri = "https://example.com/callback"
    scope = ("organizations.read",)

    def set_token(x):
        logger.info("Storing token: %s", x)

    client = Client()
    client.setup_oauth(
        client_id=client_id,
        client_secret=client_secret,
        redirect_uri=redirect_uri,
        scope=scope,
        token=oauth_token,
        set_token=set_token,
    )
    assert client._oauth_client.authorized is True
    return client


class ImprovedRequestsMock(responses.RequestsMock):
    """Wrapper adding a few shorthands to responses.RequestMock."""

    def get(self, url, filename, status=200):
        """Setup a mock response for a GET request."""
        body = self._get_body(filename)
        self.add(responses.GET, url, body=body, status=status, content_type='application/hal+json')

    def post(self, url, filename, status=200):
        """Setup a mock response for a POST request."""
        body = self._get_body(filename)
        self.add(responses.POST, url, body=body, status=status, content_type='application/hal+json')

    def delete(self, url, filename, status=204):
        """Setup a mock response for a DELETE request."""
        body = self._get_body(filename)
        self.add(responses.DELETE, url, body=body, status=status, content_type='application/hal+json')

    def patch(self, url, filename, status=200):
        """Setup a mock response for a PATCH request."""
        body = self._get_body(filename)
        self.add(responses.PATCH, url, body=body, status=status, content_type='application/hal+json')

    def _get_body(self, filename):
        """Read the response fixture file and return it."""
        file = os.path.join(os.path.dirname(__file__), 'responses', '%s.json' % filename)
        return open(file).read()


@pytest.fixture
def response():
    """Setup the responses fixture."""
    with ImprovedRequestsMock() as mock:
        yield mock



from mollie.api.objects.method import Method

from .utils import assert_list_object

PROFILE_ID = 'pfl_v9hTwCvYqw'


def test_get_profile_methods_by_profile_id(client, response):
    """Get methods relevant to profile by profile id."""
    response.get('https://api.mollie.com/v2/methods?profileId=%s' % PROFILE_ID, 'methods_list')

    methods = client.profile_methods.with_parent_id(PROFILE_ID).list()
    assert_list_object(methods, Method)



from mollie.api.objects.capture import Capture
from mollie.api.objects.payment import Payment
from mollie.api.objects.shipment import Shipment
from mollie.api.resources.captures import Captures

from .utils import assert_list_object

PAYMENT_ID = 'tr_7UhSN1zuXS'
CAPTURE_ID = 'cpt_4qqhO89gsT'
SHIPMENT_ID = 'shp_3wmsgCJN4U'


def test_capture_resource_class(client, response):
    response.get('https://api.mollie.com/v2/payments/%s/captures/%s' % (PAYMENT_ID, CAPTURE_ID),
                 'capture_single')
    client.captures.with_parent_id(PAYMENT_ID).get(CAPTURE_ID)
    assert isinstance(Capture.get_resource_class(client), Captures)


def test_get_payment_captures_by_payment_id(client, response):
    """Get capture relevant to payment by payment id."""
    response.get('https://api.mollie.com/v2/payments/%s/captures' % PAYMENT_ID, 'captures_list')

    captures = client.captures.with_parent_id(PAYMENT_ID).list()
    assert_list_object(captures, Capture)


def test_get_single_payment_capture(client, response):
    """Get a single capture relevant to payment by payment id."""
    response.get('https://api.mollie.com/v2/payments/%s/captures/%s' % (PAYMENT_ID, CAPTURE_ID),
                 'capture_single')

    capture = client.captures.with_parent_id(PAYMENT_ID).get(CAPTURE_ID)
    assert isinstance(capture, Capture)
    assert capture.id == CAPTURE_ID
    assert capture.mode == "live"
    assert capture.amount == {'currency': 'EUR', 'value': '1027.99'}
    assert capture.settlement_amount == {'currency': 'EUR', 'value': '399.00'}
    assert capture.created_at == '2018-08-02T09:29:56+00:00'
    assert capture.payment_id == PAYMENT_ID
    assert capture.shipment_id == 'shp_3wmsgCJN4U'
    assert capture.settlement_id == 'stl_jDk30akdN'


def test_list_payment_captures_by_payment_object(client, response):
    """Get a list of capture relevant to payment object."""
    response.get('https://api.mollie.com/v2/payments/%s/captures' % PAYMENT_ID, 'captures_list')
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')
    payment = client.payments.get(PAYMENT_ID)
    captures = client.captures.on(payment).list()
    assert_list_object(captures, Capture)


def test_get_single_payment_capture_by_payment_object(client, response):
    """Get a single capture relevant to payment object."""
    response.get('https://api.mollie.com/v2/payments/%s/captures/%s' % (PAYMENT_ID, CAPTURE_ID),
                 'capture_single')
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')

    payment = client.payments.get(PAYMENT_ID)
    capture = client.captures.on(payment).get(CAPTURE_ID)
    assert isinstance(capture, Capture)
    assert capture.payment_id == PAYMENT_ID


def test_capture_get_related_payment(client, response):
    """Verify the related payment of a capture."""
    #
    response.get('https://api.mollie.com/v2/payments/%s/captures/%s' % (PAYMENT_ID, CAPTURE_ID),
                 'capture_single')
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')

    capture = client.captures.with_parent_id(PAYMENT_ID).get(CAPTURE_ID)
    payment = capture.payment
    assert isinstance(payment, Payment)
    assert payment.id == PAYMENT_ID


def test_capture_get_related_shipment(client, response):
    """Verify the related shipment of a capture."""

    response.get('https://api.mollie.com/v2/payments/%s/captures/%s' % (PAYMENT_ID, CAPTURE_ID),
                 'capture_single')
    response.get('https://api.mollie.com/v2/orders/ord_8wmqcHMN4U/shipments/shp_3wmsgCJN4U', 'shipment_single')

    capture = client.captures.with_parent_id(PAYMENT_ID).get(CAPTURE_ID)
    shipment = capture.shipment
    assert isinstance(shipment, Shipment)
    assert shipment.id == SHIPMENT_ID



from mollie.api.objects.capture import Capture

from .utils import assert_list_object

PAYMENT_ID = 'tr_7UhSN1zuXS'
SETTLEMENT_ID = 'stl_jDk30akdN'
CHARGEBACK_ID = 'chb_n9z0tp'


def test_get_settlement_captures_by_capture_id(oauth_client, response):
    """Get captures relevant to settlement by settlement id."""
    response.get('https://api.mollie.com/v2/settlements/%s/captures' % SETTLEMENT_ID, 'captures_list')

    captures = oauth_client.settlement_captures.with_parent_id(SETTLEMENT_ID).list()
    assert_list_object(captures, Capture)


def test_list_settlement_captures_by_capture_object(oauth_client, response):
    """Get a list of captures relevant to settlement object."""
    response.get('https://api.mollie.com/v2/settlements/%s/captures' % SETTLEMENT_ID, 'captures_list')
    response.get('https://api.mollie.com/v2/settlements/%s' % SETTLEMENT_ID, 'settlement_single')

    settlement = oauth_client.settlements.get(SETTLEMENT_ID)
    captures = oauth_client.settlement_captures.on(settlement).list()
    assert_list_object(captures, Capture)



from mollie.api.objects.method import Method

from .utils import assert_list_object


def test_list_methods(client, response):
    """Retrieve a list of available payment methods."""
    response.get('https://api.mollie.com/v2/methods', 'methods_list')

    methods = client.methods.list()
    assert_list_object(methods, Method)


def test_method_get(client, response):
    """Retrieve a single payment method by ID."""
    response.get('https://api.mollie.com/v2/methods/ideal', 'method_get_ideal')

    method = client.methods.get('ideal')
    assert isinstance(method, Method)
    assert method.id == Method.IDEAL
    assert method.minimum_amount == {'currency': 'EUR', 'value': '0.01'}
    assert method.maximum_amount == {'currency': 'EUR', 'value': '50000.00'}
    assert method.description == 'iDEAL'
    assert method.image_svg == 'https://www.mollie.com/external/icons/payment-methods/ideal.svg'
    assert method.image_size1x == 'https://www.mollie.com/external/icons/payment-methods/ideal.png'
    assert method.image_size2x == 'https://www.mollie.com/external/icons/payment-methods/ideal%402x.png'
    assert method.pricing == [{
        'description': 'The Netherlands',
        'fixed': {'value': '0.29', 'currency': 'EUR'},
        'variable': '0'
    }]


def test_method_get_missing_images(client, response):
    """Ensure that retrieving image URLs doesn't break when URLs are missing."""
    response.get('https://api.mollie.com/v2/methods/ideal', 'method_get_ideal_wrong_images')

    method = client.methods.get('ideal')
    assert method.image_svg is None
    assert method.image_size1x is None
    assert method.image_size2x is None


def test_list_all_methods(client, response):
    """List all payment methods that Mollie is offering, including inactive methods."""
    response.get('https://api.mollie.com/v2/methods/all', 'methods_list')

    methods = client.methods.all()
    assert_list_object(methods, Method)



from mollie.api.objects.chargeback import Chargeback

from .utils import assert_list_object

PROFILE_ID = 'pfl_v9hTwCvYqw'


def test_get_profile_chargebacks_by_profile_id(client, response):
    """Get chargebacks relevant to profile by profile id."""
    response.get('https://api.mollie.com/v2/chargebacks?profileId=%s' % PROFILE_ID, 'chargebacks_list')

    chargebacks = client.profile_chargebacks.with_parent_id(PROFILE_ID).list()
    assert_list_object(chargebacks, Chargeback)



from mollie.api.objects.refund import Refund
from mollie.api.objects.settlement import Settlement

from .utils import assert_list_object

SETTLEMENT_ID = 'stl_jDk30akdN'


def test_list_refunds_on_settlement_object(oauth_client, response):
    """Retrieve a list of payment refunds of a payment."""
    response.get('https://api.mollie.com/v2/settlements/%s' % SETTLEMENT_ID, 'settlement_single')
    response.get('https://api.mollie.com/v2/settlements/%s/refunds' % SETTLEMENT_ID, 'refunds_list')

    settlement = oauth_client.settlements.get(SETTLEMENT_ID)
    assert isinstance(settlement, Settlement)

    refunds = oauth_client.settlement_refunds.on(settlement).list()
    assert_list_object(refunds, Refund)



from mollie.api.objects.refund import Refund

from .utils import assert_list_object

PROFILE_ID = 'pfl_v9hTwCvYqw'


def test_get_profile_refunds_by_profile_id(client, response):
    """Get refunds relevant to profile by profile id."""
    response.get('https://api.mollie.com/v2/refunds?profileId=%s' % PROFILE_ID, 'refunds_list')

    refunds = client.profile_refunds.with_parent_id(PROFILE_ID).list()
    assert_list_object(refunds, Refund)



import pytest

from mollie.api.objects.subscription import Subscription

from .utils import assert_list_object


def test_list_customers(client, response):
    """Retrieve a list of existing subscriptions."""
    response.get('https://api.mollie.com/v2/subscriptions', 'subscriptions_list')

    subscriptions = client.subscriptions.list()
    assert_list_object(subscriptions, Subscription)


def test_create_subscription(client):
    with pytest.raises(NotImplementedError, match='The endpoint "create" is not supported.'):
        client.subscriptions.create()


def test_get_subscription(client):
    with pytest.raises(NotImplementedError, match='The endpoint "get" is not supported.'):
        client.subscriptions.get('sub_rVKGtNd6s3')


def test_update_subscription(client):
    with pytest.raises(NotImplementedError, match='The endpoint "update" is not supported.'):
        client.subscriptions.update('sub_rVKGtNd6s3')


def test_delete_subscription(client):
    with pytest.raises(NotImplementedError, match='The endpoint "delete" is not supported.'):
        client.subscriptions.delete('sub_rVKGtNd6s3')



from mollie.api.objects.organization import Organization

ORGANIZATION_ID = 'org_12345678'


def test_get_organization(oauth_client, response):
    """Retrieve a single organization."""
    response.get('https://api.mollie.com/v2/organizations/%s' % ORGANIZATION_ID, 'organization_single')

    organization = oauth_client.organizations.get(ORGANIZATION_ID)
    assert isinstance(organization, Organization)
    assert organization.id == ORGANIZATION_ID
    assert organization.name == 'Mollie B.V.'
    assert organization.email == 'info@mollie.com'
    assert organization.vat_number == 'NL815839091B01'
    assert organization.registration_number == '30204462'
    assert organization.address == {
        'city': 'Amsterdam',
        'country': 'NL',
        'postalCode': '1016 EE',
        'streetAndNumber': 'Keizersgracht 313'
    }


def test_get_current_organization(oauth_client, response):
    """Retrieve a current organization."""
    response.get('https://api.mollie.com/v2/organizations/me', 'organization_current')

    organization = oauth_client.organizations.get('me')
    assert isinstance(organization, Organization)
    assert organization.id == ORGANIZATION_ID
    assert organization.name == 'Mollie B.V.'
    assert organization.email == 'info@mollie.com'
    assert organization.vat_number == 'NL815839091B01'
    assert organization.registration_number == '30204462'
    assert organization.address == {
        'city': 'Amsterdam',
        'country': 'NL',
        'postalCode': '1016 EE',
        'streetAndNumber': 'Keizersgracht 313'
    }
    assert organization.dashboard == 'https://mollie.com/dashboard/org_12345678'



from .base import Base


class Invoice(Base):
    @classmethod
    def get_resource_class(cls, client):
        from ..resources.invoices import Invoices
        return Invoices(client)

    @property
    def id(self):
        return self._get_property('id')

    @property
    def reference(self):
        return self._get_property('reference')

    @property
    def vat_number(self):
        return self._get_property('vatNumber')

    @property
    def status(self):
        return self._get_property('status')

    @property
    def issued_at(self):
        return self._get_property('issuedAt')

    @property
    def due_at(self):
        return self._get_property('dueAt')

    @property
    def resource(self):
        return self._get_property('resource')

    @property
    def net_amount(self):
        return self._get_property('netAmount')

    @property
    def vat_amount(self):
        return self._get_property('vatAmount')

    @property
    def gross_amount(self):
        return self._get_property('grossAmount')

    @property
    def lines(self):
        return self._get_property('lines') or []

    @property
    def pdf(self):
        return self._get_link('pdf')



from mollie.api.objects.refund import Refund

from .utils import assert_list_object


def test_list_refunds(client, response):
    """Retrieve a list of refunds."""
    response.get('https://api.mollie.com/v2/refunds', 'refunds_list')

    refunds = client.refunds.list()
    assert_list_object(refunds, Refund)



from mollie.api.objects.chargeback import Chargeback

from .utils import assert_list_object


def test_list_chargebacks(client, response):
    """Get a list of chargebacks."""
    response.get('https://api.mollie.com/v2/chargebacks', 'chargebacks_list')

    chargebacks = client.chargebacks.list()
    assert_list_object(chargebacks, Chargeback)



from .base import Base


class OrderLine(Base):
    @classmethod
    def get_resource_class(cls, client):
        from ..resources.order_lines import OrderLines
        return OrderLines(client)

    STATUS_CREATED = 'created'
    STATUS_AUTHORIZED = 'authorized'
    STATUS_PAID = 'paid'
    STATUS_SHIPPING = 'shipping'
    STATUS_CANCELED = 'canceled'
    STATUS_COMPLETED = 'completed'

    @classmethod
    def get_object_name(cls):
        return 'lines'

    @property
    def id(self):
        return self._get_property('id')

    @property
    def resource(self):
        return self._get_property('resource')

    @property
    def order_id(self):
        return self._get_property('orderId')

    @property
    def type(self):
        return self._get_property('type')

    @property
    def name(self):
        return self._get_property('name')

    @property
    def status(self):
        return self._get_property('status')

    @property
    def is_cancelable(self):
        return self._get_property('isCancelable')

    @property
    def quantity(self):
        return self._get_property('quantity')

    @property
    def quantity_shipped(self):
        return self._get_property('quantityShipped')

    @property
    def amount_shipped(self):
        return self._get_property('amountShipped')

    @property
    def quantity_refunded(self):
        return self._get_property('quantityRefunded')

    @property
    def amount_refunded(self):
        return self._get_property('amountRefunded')

    @property
    def quantity_canceled(self):
        return self._get_property('quantityCanceled')

    @property
    def amount_canceled(self):
        return self._get_property('amountCanceled')

    @property
    def shippable_quantity(self):
        return self._get_property('shippableQuantity')

    @property
    def refundable_quantity(self):
        return self._get_property('refundableQuantity')

    @property
    def cancelable_quantity(self):
        return self._get_property('cancelableQuantity')

    @property
    def unit_price(self):
        return self._get_property('unitPrice')

    @property
    def discount_amount(self):
        return self._get_property('discountAmount')

    @property
    def total_amount(self):
        return self._get_property('totalAmount')

    @property
    def vat_rate(self):
        return self._get_property('vatRate')

    @property
    def vat_amount(self):
        return self._get_property('vatAmount')

    @property
    def sku(self):
        return self._get_property('sku')

    @property
    def image_url(self):
        return self._get_link('imageUrl')

    @property
    def product_url(self):
        return self._get_link('productUrl')

    @property
    def created_at(self):
        return self._get_property('createdAt')

    @property
    def metadata(self):
        return self._get_property('metadata')

    # additional methods

    def is_created(self):
        return self._get_property('status') == self.STATUS_CREATED

    def is_authorized(self):
        return self._get_property('status') == self.STATUS_AUTHORIZED

    def is_paid(self):
        return self._get_property('status') == self.STATUS_PAID

    def is_shipping(self):
        return self._get_property('status') == self.STATUS_SHIPPING

    def is_canceled(self):
        return self._get_property('status') == self.STATUS_CANCELED

    def is_completed(self):
        return self._get_property('status') == self.STATUS_COMPLETED



import pytest

from mollie.api.error import IdentifierError
from mollie.api.objects.permission import Permission
from mollie.api.resources.permissions import Permissions

from .utils import assert_list_object

PERMISSION_ID = 'payments.read'


def test_list_permissions(oauth_client, response):
    """Retrieve a list of existing permissions."""
    response.get('https://api.mollie.com/v2/permissions', 'permissions_list')

    permissions = oauth_client.permissions.list()
    assert_list_object(permissions, Permission)


def test_get_permission(oauth_client, response):
    """Retrieve a single permission."""
    response.get('https://api.mollie.com/v2/permissions/%s' % PERMISSION_ID, 'permission_single')

    permission = oauth_client.permissions.get(PERMISSION_ID)
    assert isinstance(permission, Permission)
    assert permission.id == PERMISSION_ID
    assert permission.resource == 'permission'
    assert permission.description == "View your payments"
    assert permission.granted is True


@pytest.mark.parametrize(
    "input,expected",
    [
        (None, IdentifierError),
        ("foo", IdentifierError),
        ("foo.", IdentifierError),
        (".bar", IdentifierError),
        ("foo1", IdentifierError),
        ("foo1.", IdentifierError),
        (".bar1", IdentifierError),
        ("foo.bar1", IdentifierError),
        ("foo1.bar", IdentifierError),
        ("foo.bar", None),  # Valid
    ]
)
def test_validate_permission_id(input, expected):
    if expected == IdentifierError:
        with pytest.raises(IdentifierError):
            Permissions.validate_permission_id(input)
    else:
        assert Permissions.validate_permission_id(input) is expected



from mollie.api.objects.onboarding import Onboarding
from mollie.api.objects.organization import Organization

ORGANIZATION_ID = 'org_12345678'


def test_get_onboarding(oauth_client, response):
    """Retrieve onboarding."""
    response.get('https://api.mollie.com/v2/onboarding/me', 'onboarding_me')

    onboarding = oauth_client.onboarding.get('me')
    assert isinstance(onboarding, Onboarding)

    assert onboarding.name == 'Mollie B.V.'
    assert onboarding.signed_up_at == '2018-12-20T10:49:08+00:00'
    assert onboarding.status == 'completed'
    assert onboarding.can_receive_payments is True
    assert onboarding.can_receive_settlements is True

    assert onboarding.is_needs_data() is False
    assert onboarding.is_in_review() is False
    assert onboarding.is_completed() is True


def test_create_onboarding(oauth_client, response):
    """Create onboarding."""
    response.post('https://api.mollie.com/v2/onboarding/me', 'empty', 204)

    data = {
        'profile': {
            'categoryCode': '6012'
        }
    }

    onboarding = oauth_client.onboarding.create(resource_id='me', data=data)
    assert isinstance(onboarding, Onboarding)


def test_onboarding_get_organization(oauth_client, response):
    """Retrieve organization related to onboarding."""
    response.get('https://api.mollie.com/v2/onboarding/me', 'onboarding_me')
    response.get('https://api.mollie.com/v2/organization/%s' % ORGANIZATION_ID, 'organization_single')

    onboarding = oauth_client.onboarding.get('me')
    organization = onboarding.organization
    assert isinstance(organization, Organization)
    assert organization.id == ORGANIZATION_ID



from ..error import IdentifierError
from ..objects.capture import Capture
from .base import Base


class Captures(Base):
    RESOURCE_ID_PREFIX = 'cpt_'

    def get_resource_object(self, result):
        return Capture(result, self.client)

    def get_resource_name(self):
        return 'payments/{id}/captures'.format(id=self.payment_id)

    def get(self, capture_id, **params):
        """Verify the capture ID and retrieve the capture from the API."""
        if not capture_id or not capture_id.startswith(self.RESOURCE_ID_PREFIX):
            raise IdentifierError(
                "Invalid capture ID: '{id}'. A capture ID should start with '{prefix}'.".format(
                    id=capture_id, prefix=self.RESOURCE_ID_PREFIX)
            )
        return super().get(capture_id, **params)

    def with_parent_id(self, payment_id):
        self.payment_id = payment_id
        return self

    def on(self, payment):
        return self.with_parent_id(payment.id)



import re

from ..error import IdentifierError
from ..objects.settlement import Settlement
from .base import Base


class Settlements(Base):
    RESOURCE_ID_PREFIX = 'stl_'

    # According to Mollie, the bank reference is formatted as:
    # - The Mollie customer ID, 4 to 7 digits.
    # - The year and month, 4 digits
    # - The sequence number of the settlement in that month, 2 digits
    # The components are separated by a dot.
    BANK_REFERENCE_REGEX = re.compile(r'^\d{4,7}\.\d{4}\.\d{2}$', re.ASCII)

    def get_resource_object(self, result):
        return Settlement(result, self.client)

    @classmethod
    def validate_settlement_id(cls, settlement_id):
        if not settlement_id or (
            not settlement_id.startswith(cls.RESOURCE_ID_PREFIX) and
            settlement_id not in ['next', 'open'] and
            not cls.BANK_REFERENCE_REGEX.match(settlement_id)
        ):
            raise IdentifierError(
                "Invalid settlement ID: '{id}'. A settlement ID should start with '{prefix}' "
                ", be 'next' or 'open' or contain a valid bank reference.".format(
                    id=settlement_id, prefix=cls.RESOURCE_ID_PREFIX)
            )

    def get(self, settlement_id, **params):
        self.validate_settlement_id(settlement_id)
        """Verify the settlement ID and retrieve the settlement from the API."""
        return super().get(settlement_id, **params)



from .refunds import Refunds


class ProfileRefunds(Refunds):
    profile_id = None

    def get_resource_name(self):
        return 'refunds?profileId={id}'.format(id=self.profile_id)

    def with_parent_id(self, profile_id):
        self.profile_id = profile_id
        return self

    def on(self, profile):
        return self.with_parent_id(profile.id)



import pytest

from mollie.api.error import IdentifierError
from mollie.api.objects.customer import Customer
from mollie.api.objects.method import Method
from mollie.api.objects.payment import Payment
from mollie.api.objects.subscription import Subscription

from .utils import assert_list_object

CUSTOMER_ID = 'cst_8wmqcHMN4U'
SUBSCRIPTION_ID = 'sub_rVKGtNd6s3'


def test_list_customer_subscriptions(client, response):
    """Retrieve a list of subscriptions."""
    response.get('https://api.mollie.com/v2/customers/%s/subscriptions' % CUSTOMER_ID, 'subscriptions_customer_list')

    subscriptions = client.customer_subscriptions.with_parent_id(CUSTOMER_ID).list()
    assert_list_object(subscriptions, Subscription)


def test_get_customer_subscription_by_id(client, response):
    """Retrieve a single subscription by ID."""
    response.get('https://api.mollie.com/v2/customers/%s/subscriptions/%s' % (CUSTOMER_ID, SUBSCRIPTION_ID),
                 'subscription_single')
    response.get('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_single')

    subscription = client.customer_subscriptions.with_parent_id(CUSTOMER_ID).get(SUBSCRIPTION_ID)
    assert subscription.resource == 'subscription'
    assert subscription.id == SUBSCRIPTION_ID
    assert subscription.mode == 'live'
    assert subscription.created_at == '2016-06-01T12:23:34+00:00'
    assert subscription.amount == {'value': '25.00', 'currency': 'EUR'}
    assert subscription.times == 4
    assert subscription.interval == '3 months'
    assert subscription.description == 'Quarterly payment'
    assert subscription.method == Method.IDEAL
    assert subscription.webhook_url == 'https://webshop.example.org/payments/webhook'
    assert subscription.status == Subscription.STATUS_ACTIVE
    assert subscription.start_date is None
    assert subscription.canceled_at is None
    assert subscription.customer is not None
    assert subscription.metadata == {'order_id': 1337}
    assert subscription.is_active() is True
    assert subscription.is_suspended() is False
    assert subscription.is_pending() is False
    assert subscription.is_completed() is False
    assert subscription.is_canceled() is False


def test_list_customer_subscriptions_by_customer_object(client, response):
    """Retrieve a list of subscriptions related to customer."""
    response.get('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_single')
    response.get('https://api.mollie.com/v2/customers/%s/subscriptions' % CUSTOMER_ID,
                 'subscriptions_customer_list')

    customer = client.customers.get(CUSTOMER_ID)
    subscriptions = client.customer_subscriptions.on(customer).list()
    assert_list_object(subscriptions, Subscription)


def test_get_customer_subscription_by_customer_object(client, response):
    """Retrieve specific subscription related to customer."""
    response.get('https://api.mollie.com/v2/customers/%s/subscriptions/%s' % (CUSTOMER_ID, SUBSCRIPTION_ID),
                 'subscription_single')
    response.get('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_single')

    customer = client.customers.get(CUSTOMER_ID)
    subscription = client.customer_subscriptions.on(customer).get(SUBSCRIPTION_ID)
    assert isinstance(subscription, Subscription)
    assert subscription.customer.id == CUSTOMER_ID


def test_customer_subscription_get_related_customer(client, response):
    """Retrieve a related customer object from a subscription."""
    response.get('https://api.mollie.com/v2/customers/%s/subscriptions/%s' % (CUSTOMER_ID, SUBSCRIPTION_ID),
                 'subscription_single')
    response.get('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_single')

    subscription = client.customer_subscriptions.with_parent_id(CUSTOMER_ID).get(SUBSCRIPTION_ID)
    assert isinstance(subscription.customer, Customer)
    assert subscription.customer.id == CUSTOMER_ID


def test_cancel_customer_subscription(client, response):
    """Cancel a subscription by customer ID and subscription ID."""
    response.delete('https://api.mollie.com/v2/customers/%s/subscriptions/%s' % (CUSTOMER_ID, SUBSCRIPTION_ID),
                    'subscription_canceled', 200)

    subscription = client.customer_subscriptions.with_parent_id(CUSTOMER_ID).delete(SUBSCRIPTION_ID)
    assert isinstance(subscription, Subscription)
    assert subscription.status == 'canceled'
    assert subscription.canceled_at == '2018-08-01T11:04:21+00:00'


def test_cancel_customer_subscription_invalid_id(client):
    """Verify that an invalid subscription id is validated and raises an error."""
    with pytest.raises(IdentifierError):
        client.customer_subscriptions.with_parent_id(CUSTOMER_ID).delete('invalid')


def test_create_customer_subscription(client, response):
    """Create a subscription with customer object."""
    response.post('https://api.mollie.com/v2/customers/%s/subscriptions' % CUSTOMER_ID, 'subscription_single')

    data = {
        'amount': {'currency': 'EUR', 'value': '25.00'},
        'times': 4,
        'interval': '3 months',
        'description': 'Quarterly payment',
        'webhookUrl': 'https://webshop.example.org/subscriptions/webhook'
    }
    subscription = client.customer_subscriptions.with_parent_id(CUSTOMER_ID).create(data=data)
    assert isinstance(subscription, Subscription)
    assert subscription.id == SUBSCRIPTION_ID


def test_update_customer_subscription(client, response):
    """Update existing subscription of a customer."""
    response.patch('https://api.mollie.com/v2/customers/%s/subscriptions/%s' % (CUSTOMER_ID, SUBSCRIPTION_ID),
                   'subscription_updated')

    data = {
        'amount': {'currency': 'USD', 'value': '30.00'},
        'times': 42,
        'startDate': '2018-12-12',
        'description': 'Updated subscription',
        'webhookUrl': 'https://webshop.example.org/subscriptions/webhook'
    }
    subscription = client.customer_subscriptions.with_parent_id(CUSTOMER_ID).update(SUBSCRIPTION_ID, data)
    assert isinstance(subscription, Subscription)
    assert subscription.id == SUBSCRIPTION_ID


def test_customer_subscription_get_related_payments(client, response):
    """Retrieve a list of payments related to the subscription. """
    response.get('https://api.mollie.com/v2/customers/%s/subscriptions/%s' % (CUSTOMER_ID, SUBSCRIPTION_ID),
                 'subscription_single')
    response.get('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_single')
    response.get('https://api.mollie.com/v2/customers/%s/subscriptions/%s/payments' % (CUSTOMER_ID, SUBSCRIPTION_ID),
                 'payments_list')
    subscription = client.customer_subscriptions.with_parent_id(CUSTOMER_ID).get(SUBSCRIPTION_ID)
    payments = subscription.payments
    assert_list_object(payments, Payment)



from mollie.api.objects.order_line import OrderLine

from .utils import assert_list_object

ORDER_ID = 'ord_kEn1PlbGa'
LINE_ID = 'odl_dgtxyl'


def test_get_order_lines(client, response):
    """Retrieve lines of a single order by order ID."""
    response.get('https://api.mollie.com/v2/orders/{order_id}'.format(order_id=ORDER_ID), 'order_single')

    order = client.orders.get(ORDER_ID)
    lines = order.lines
    assert_list_object(lines, OrderLine)

    # Test properties of the first line
    line = next(lines)
    assert isinstance(line, OrderLine)
    assert line.id == LINE_ID
    assert line.resource == 'orderline'
    assert line.order_id == ORDER_ID
    assert line.type == 'physical'
    assert line.name == 'LEGO 42083 Bugatti Chiron'
    assert line.status == 'created'
    assert line.is_cancelable is True
    assert line.quantity == 2
    assert line.shippable_quantity == 0
    assert line.quantity_shipped == 0
    assert line.amount_shipped == {'value': '0.00', 'currency': 'EUR'}
    assert line.refundable_quantity == 0
    assert line.quantity_refunded == 0
    assert line.amount_refunded == {'value': '0.00', 'currency': 'EUR'}
    assert line.cancelable_quantity == 0
    assert line.quantity_canceled == 0
    assert line.amount_canceled == {'value': '0.00', 'currency': 'EUR'}
    assert line.unit_price == {'value': '399.00', 'currency': 'EUR'}
    assert line.discount_amount == {'value': '100.00', 'currency': 'EUR'}
    assert line.total_amount == {'value': '698.00', 'currency': 'EUR'}
    assert line.vat_rate == '21.00'
    assert line.vat_amount == {'value': '121.14', 'currency': 'EUR'}
    assert line.sku == '5702016116977'
    assert line.image_url == 'https://sh-s7-live-s.legocdn.com/is/image//LEGO/42083_alt1?$main$'
    assert line.product_url == 'https://shop.lego.com/nl-NL/Bugatti-Chiron-42083'
    assert line.created_at == '2018-08-02T09:29:56+00:00'
    assert line.metadata == {'brickwatch_url': 'https://www.brickwatch.net/nl-NL/set/42083/Bugatti-Chiron.html'}
    assert line.is_created() is True
    assert line.is_authorized() is False
    assert line.is_paid() is False
    assert line.is_shipping() is False
    assert line.is_canceled() is False
    assert line.is_completed() is False


def test_update_order_line(client, response):
    """Update a line by id through an order object."""
    response.get('https://api.mollie.com/v2/orders/{order_id}'.format(order_id=ORDER_ID), 'order_single')
    response.patch('https://api.mollie.com/v2/orders/{order_id}/lines/{order_line_id}'.format(
        order_id=ORDER_ID, order_line_id=LINE_ID), 'order_single')
    data = {
        "name": "LEGO 71043 Hogwarts Castle",
        "productUrl": "https://shop.lego.com/en-GB/product/Hogwarts-Castle-71043",
        "imageUrl": "https://sh-s7-live-s.legocdn.com/is/image//LEGO/71043_alt1?$main$",
        "quantity": 2,
        "sku": "335596985",
        "vatRate": "21.00",
        "unitPrice": {
            "currency": "EUR",
            "value": "349.00"
        },
        "totalAmount": {
            "currency": "EUR",
            "value": "598.00"
        },
        "discountAmount": {
            "currency": "EUR",
            "value": "100.00"
        },
        "vatAmount": {
            "currency": "EUR",
            "value": "103.79"
        }
    }
    order = client.orders.get(ORDER_ID)
    update = order.update_line(LINE_ID, data)
    assert isinstance(update, OrderLine)



from .base import Base


class Issuer(Base):

    @property
    def resource(self):
        return self._get_property('resource')

    @property
    def id(self):
        return self._get_property('id')

    @property
    def name(self):
        return self._get_property('name')

    @property
    def image_svg(self):
        try:
            images = self._get_property('image')
            return images['svg']
        except KeyError:
            return None

    @property
    def image_size1x(self):
        try:
            images = self._get_property('image')
            return images['size1x']
        except KeyError:
            return None

    @property
    def image_size2x(self):
        try:
            images = self._get_property('image')
            return images['size2x']
        except KeyError:
            return None



import re
import sys
import time
from datetime import datetime

import pytest
import requests.adapters

from mollie.api.client import Client, generate_querystring
from mollie.api.error import (
    DataConsistencyError,
    IdentifierError,
    NotFoundError,
    RequestError,
    RequestSetupError,
    ResponseError,
    ResponseHandlingError,
    UnauthorizedError,
    UnprocessableEntityError,
)
from mollie.api.objects.method import Method
from mollie.api.objects.organization import Organization

from .utils import assert_list_object


@pytest.mark.parametrize('params, querystring', [
    ({}, None),
    ({'locale': 'nl_NL'}, 'locale=nl_NL'),
    ({'locale': 'nl_NL', 'hoeba': 'kek'}, 'hoeba=kek&locale=nl_NL'),
    ({'amount': {'value': '100.00', 'currency': 'USD'}}, 'amount%5Bcurrency%5D=USD&amount%5Bvalue%5D=100.00')
])
def test_generate_querystring(params, querystring):
    """Verify that we can generate querystring that are correctly quoted."""
    result = generate_querystring(params)
    assert result == querystring


def test_client_querystring(client, response):
    """Verify that we are triggering the correct URL when using querystring with square brackets."""
    response.add(
        response.GET,
        'https://api.mollie.com/v2/methods?amount[currency]=USD&amount[value]=100.00',
        body=response._get_body('methods_list'),
        match_querystring=True
    )

    params = {'amount': {'currency': 'USD', 'value': '100.00'}}
    methods = client.methods.list(**params)
    assert_list_object(methods, Method)


def test_client_api_key():
    """Setting up a valid api key or access token should be possible."""
    client = Client()

    client.set_access_token('access_123')
    assert client.api_key == 'access_123'

    client.set_api_key('live_123')
    assert client.api_key == 'live_123'

    client.set_api_key('test_123')
    assert client.api_key == 'test_123'


def test_client_no_api_key():
    """A Request without an API key should raise an error."""
    client = Client()
    with pytest.raises(RequestSetupError, match='You have not set an API key.'):
        client.customers.list()


def test_client_invalid_api_key():
    """Setting up an invalid api key raises an error."""
    client = Client()

    with pytest.raises(RequestSetupError, match="Invalid API key: 'invalid'"):
        client.set_api_key('invalid')

    with pytest.raises(RequestSetupError, match="Invalid API key: 'access_123'"):
        client.set_api_key('access_123')

    with pytest.raises(RequestSetupError, match="Invalid access token: 'invalid'"):
        client.set_access_token('invalid')

    with pytest.raises(RequestSetupError, match="Invalid access token: 'live_123'"):
        client.set_access_token('live_123')

    with pytest.raises(RequestSetupError, match="Invalid access token: 'test_123'"):
        client.set_access_token('test_123')


def test_client_broken_cert_bundle(monkeypatch):
    """A request should raise an error when the certificate bundle is not available.

    Under circumstances it could be possible that the certifi package is not correctly installed, broken,
    or just plain too old. Connecting to the Mollie API should fail with an error when the certificate
    cannot be verified.
    """
    monkeypatch.setenv("REQUESTS_CA_BUNDLE", "/does/not/exist")

    client = Client()
    client.set_api_key('test_test')
    with pytest.raises(OSError) as excinfo:
        client.customers.list()
    assert 'Could not find a suitable TLS CA certificate bundle, invalid path: /does/not/exist' in str(excinfo.value)


def test_client_generic_request_error(response):
    """When the remote server refuses connections or other request issues arise, an error should be raised.

    The 'response' fixture blocks all outgoing connections, also when no actual responses are configured.
    """
    client = Client()
    client.set_api_key('test_test')
    client.set_api_endpoint('https://api.mollie.invalid/')
    with pytest.raises(RequestError, match='Unable to communicate with Mollie: Connection refused'):
        client.customers.list()


def test_client_invalid_create_data(client):
    """Invalid data for a create command should raise an error."""
    data = datetime.now()
    with pytest.raises(RequestSetupError, match='Error encoding parameters into JSON'):
        client.customers.create(data=data)


def test_client_invalid_update_data(client):
    """Invalid data for a create command should raise an error."""
    data = datetime.now()
    with pytest.raises(RequestSetupError, match='Error encoding parameters into JSON'):
        client.customers.update('cst_12345', data=data)


@pytest.mark.parametrize('endpoint, errorstr', [
    ('customers', "Invalid customer ID: 'invalid'. A customer ID should start with 'cst_'."),
    ('payments', "Invalid payment ID: 'invalid'. A payment ID should start with 'tr_'."),
    ('refunds', "Invalid refund ID: 'invalid'. A refund ID should start with 're_'."),
    ('orders', "Invalid order ID: 'invalid'. An order ID should start with 'ord_'."),
])
def test_client_get_invalid_id(client, endpoint, errorstr):
    """An invalid formatted object ID should raise an error."""
    with pytest.raises(IdentifierError, match=errorstr):
        getattr(client, endpoint).get('invalid')


@pytest.mark.parametrize('endpoint, errorstr', [
    ('customer_mandates', "Invalid mandate ID: 'invalid'. A mandate ID should start with 'mdt_'."),
    ('customer_payments', "Invalid payment ID: 'invalid'. A payment ID should start with 'tr_'."),
    ('customer_subscriptions', "Invalid subscription ID: 'invalid'. A subscription ID should start with 'sub_'."),
])
def test_client_get_customer_related_invalid_id(client, endpoint, errorstr):
    """An invalid formatted object ID should raise an error."""
    with pytest.raises(IdentifierError, match=errorstr):
        getattr(client, endpoint).with_parent_id('cst_12345').get('invalid')


@pytest.mark.parametrize('endpoint, errorstr', [
    ('payment_chargebacks', "Invalid chargeback ID: 'invalid'. A chargeback ID should start with 'chb_'."),
    ('payment_refunds', "Invalid refund ID: 'invalid'. A refund ID should start with 're_'."),
])
def test_client_get_payment_related_invalid_id(client, endpoint, errorstr):
    """An invalid formatted object ID should raise an error."""
    with pytest.raises(IdentifierError, match=errorstr):
        getattr(client, endpoint).with_parent_id('tr_12345').get('invalid')


def test_client_invalid_json_response(client, response):
    """An invalid json response should raise an error."""
    response.get('https://api.mollie.com/v2/customers', 'invalid_json')
    with pytest.raises(ResponseHandlingError, match=r'Unable to decode Mollie API response \(status code: 200\)'):
        client.customers.list()


@pytest.mark.parametrize('resp_payload, resp_status, exception, errormsg', [
    ('error_unauthorized', 401, UnauthorizedError, 'Missing authentication, or failed to authenticate'),
    ('customer_doesnotexist', 404, NotFoundError, 'No customer exists with token cst_doesnotexist.'),
    ('payment_rejected', 422, UnprocessableEntityError, 'The amount is higher than the maximum'),
    ('error_teapot', 418, ResponseError, 'Just an example error that is not explicitly supported'),
])
def test_client_get_received_error_response(client, response, resp_payload, resp_status, exception, errormsg):
    """An error response from the API should raise a matching error."""
    response.get('https://api.mollie.com/v2/customers/cst_doesnotexist', resp_payload, status=resp_status)
    with pytest.raises(exception, match=errormsg) as excinfo:
        client.customers.get('cst_doesnotexist')
    assert excinfo.value.status == resp_status


@pytest.mark.parametrize('resp_payload, resp_status, exception, errormsg', [
    ('error_unauthorized', 401, UnauthorizedError, 'Missing authentication, or failed to authenticate'),
    ('customer_doesnotexist', 404, NotFoundError, 'No customer exists with token cst_doesnotexist.'),
    ('error_teapot', 418, ResponseError, 'Just an example error that is not explicitly supported'),
])
def test_client_delete_received_error_response(client, response, resp_payload, resp_status, exception, errormsg):
    """When deleting, an error response from the API should raise a matching error."""
    response.delete('https://api.mollie.com/v2/customers/cst_doesnotexist', resp_payload, status=resp_status)
    with pytest.raises(exception, match=errormsg) as excinfo:
        client.customers.delete('cst_doesnotexist')
    assert excinfo.value.status == resp_status


def test_client_response_404_but_no_payload(response):
    """An error response from the API should raise an error.

    When the response returns an error, but no valid error data is available in the response,
    we should still raise an error. The API v1 formatted error in the test is missing the required 'status' field.
    """
    response.get('https://api.mollie.com/v3/customers', 'v1_api_error', status=404)
    client = Client()
    client.api_version = 'v3'
    client.set_api_key('test_test')

    with pytest.raises(ResponseHandlingError, match='Invalid API version'):
        client.customers.list()


def test_client_error_including_field_response(client, response):
    """An error response containing a 'field' value should be reflected in the raised error."""
    response.post('https://api.mollie.com/v2/payments', 'payment_rejected', status=422)
    params = {
        'amount': {
            'value': '10000000.00',
            'currency': 'EUR',
        },
        'method': 'ideal',
        'description': 'My order',
        'redirectUrl': 'https://webshop.example.org/order/12345/',
        'webhookUrl': 'https://webshop.example.org/payments/webhook/',
    }
    with pytest.raises(UnprocessableEntityError, match='The amount is higher than the maximum') as excinfo:
        client.payments.create(**params)
    assert excinfo.value.field == 'amount'


@pytest.mark.skipif(sys.version_info.major == 2, reason='output differs for python 2')
def test_client_unicode_error_py3(client, response):
    """An error response containing Unicode characters should also be processed correctly."""
    response.post('https://api.mollie.com/v2/orders', 'order_error', status=422)
    with pytest.raises(UnprocessableEntityError) as err:
        # actual POST data for creating an order can be found in test_orders.py
        client.orders.create({})

    # handling the error should work even when utf-8 characters (€) are in the response.
    exception = err.value
    expected = 'Order line 1 is invalid. VAT amount is off. ' \
               'Expected VAT amount to be €3.47 (21.00% over €20.00), got €3.10'
    assert str(exception) == expected


def test_client_request_timeout(mocker, client):
    """Mock requests.request in the client to be able to read if the timeout is in the request call args."""
    mocked_request = mocker.patch("mollie.api.client.requests.Session.request")
    # Create a mocked response for the request
    response = mocker.Mock(status_code=200)
    response.headers.get.return_value = "application/hal+json"
    response.json.return_value = {}
    mocked_request.return_value = response

    client.set_timeout(300)
    client.payments.list()
    assert mocked_request.call_args[1]['timeout'] == 300


def test_client_request_timed_out(mocker, client):
    """Timeout should raise a RequestError."""
    mocker.patch(
        "mollie.api.client.requests.Session.request",
        side_effect=requests.exceptions.ReadTimeout(
            "HTTPSConnectionPool(host='api.mollie.com', port=443): Read timed out. (read timeout=10)"
        )
    )

    with pytest.raises(RequestError, match='Read timed out.'):
        client.payments.list()


def test_client_will_propagate_retry_setting(response):
    response.get('https://api.mollie.com/v2/methods', 'methods_list')

    client = Client(retry=3)
    client.set_api_key("test_test")
    client.methods.list()

    adapter = client._client.adapters['https://']
    assert adapter.max_retries.connect == 3


def test_client_data_consistency_error(client, response):
    """When the API sends us data we did not expect raise an consistency error."""
    order_id = 'ord_kEn1PlbGa'
    line_id = 'odl_12345'
    response.get('https://api.mollie.com/v2/orders/{order_id}'.format(order_id=order_id), 'order_single')
    response.patch('https://api.mollie.com/v2/orders/{order_id}/lines/{order_line_id}'.format(
        order_id=order_id, order_line_id=line_id), 'order_single')

    order = client.orders.get(order_id)
    data = {
        "name": "LEGO 71043 Hogwarts™ Castle",
    }
    # Update an nonexistent order line. This raises an data consistency error.
    with pytest.raises(DataConsistencyError, match=r'Line id .* not found in response.'):
        order.update_line(line_id, data)


def test_client_default_user_agent(client, response):
    """Default user-agent should contain some known values."""
    regex = re.compile(r'^Mollie/[\d\.]+ Python/[\w\.\+]+ OpenSSL/[\w\.]+$')
    assert re.match(regex, client.user_agent)

    # perform a request and inpect the actual used headers
    response.get('https://api.mollie.com/v2/methods', 'methods_list')
    client.methods.list()
    request = response.calls[0].request
    assert re.match(regex, request.headers['User-Agent'])


def test_oauth_client_default_user_agent(oauth_client, response):
    """Default user-agent should contain some known values."""
    regex = re.compile(r'^Mollie/[\d\.]+ Python/[\w\.\+]+ OpenSSL/[\w\.]+ OAuth/2.0$')
    assert re.match(regex, oauth_client.user_agent)

    # perform a request and inpect the actual used headers
    response.get('https://api.mollie.com/v2/organizations/me', 'organization_current')
    oauth_client.organizations.get('me')
    request = response.calls[0].request
    assert re.match(regex, request.headers['User-Agent'])


def test_client_user_agent_with_access_token():
    """When authenticating with an access token, the User-Agent should contain an OAuth component."""
    client = Client()
    assert 'OAuth'.lower() not in client.user_agent.lower()
    client.set_access_token('access_123')
    assert 'OAuth/2.0' in client.user_agent


def test_client_set_user_agent_component(response):
    """We should be able to add useragent components.

    Note: we don't use the fixture client because it is shared between tests, and we don't want it
    to be clobbered with random User-Agent strings.
    """
    client = Client()
    assert 'Hoeba' not in client.user_agent
    client.set_user_agent_component('Hoeba', '1.0.0')
    assert 'Hoeba/1.0.0' in client.user_agent

    response.get('https://api.mollie.com/v2/methods', 'methods_list')
    client.set_api_key('test_123')
    client.methods.list()
    request = response.calls[0].request
    assert 'Hoeba/1.0.0' in request.headers['User-Agent']


@pytest.mark.parametrize("key, expected", [
    ('lowercase', 'Lowercase'),
    ('UPPERCASE', 'Uppercase'),
    ('multiple words', 'MultipleWords'),
    ('multiple   spaces', 'MultipleSpaces'),
    ('trailing space ', 'TrailingSpace')
])
def test_client_set_user_agent_component_correct_key_syntax(key, expected):
    """When we receive UA component keys that don't adhere to the proposed syntax, they are corrected."""
    client = Client()
    client.set_user_agent_component(key, '1.0.0')
    assert "{expected}/1.0.0".format(expected=expected) in client.user_agent


@pytest.mark.parametrize("value, expected", [
    ('1.2.3', '1.2.3'),
    ('singleword', 'singleword'),
    ('MiXedCaSe', 'MiXedCaSe'),  # should be preserved
    ('UPPERCASE', 'UPPERCASE'),  # should be preserved
    ('with space', 'with_space'),
    ('multiple   spaces', 'multiple_spaces'),
    ('trailing space ', 'trailing_space')
])
def test_client_set_user_agent_component_correct_value_syntax(value, expected):
    """When we receive UA component values that don't adhere to the proposed syntax, they are corrected."""
    client = Client()
    client.set_user_agent_component('Something', value)
    assert "Something/{expected}".format(expected=expected) in client.user_agent


def test_client_update_user_agent_component():
    """We should be able to update the User-Agent component when using the same key."""
    client = Client()
    client.set_user_agent_component('Test', '1.0.0')
    assert 'Test/1.0.0' in client.user_agent

    # now update the component using the same key
    client.set_user_agent_component('Test', '2.0.0')
    assert 'Test/2.0.0' in client.user_agent
    assert 'Test/1.0.0' not in client.user_agent

    # and update with a key that will be converted to the same value
    client.set_user_agent_component('TEST', '3.0.0')
    assert 'Test/3.0.0' in client.user_agent
    assert 'Test/2.0.0' not in client.user_agent
    assert 'Test/1.0.0' not in client.user_agent


def test_oauth_client_will_refresh_token_automatically(mocker, oauth_token, response):
    """Initializing the client with an expired token will trigger a token refresh automatically."""
    # expire the token: set expiration time in the past.
    oauth_token["expires_at"] = time.time() - 5

    set_token_mock = mocker.Mock()

    client = Client()
    client.setup_oauth(
        client_id="client_id",
        client_secret="client_secret",
        redirect_uri="https://example.com/callback",
        scope=("organizations.read",),
        token=oauth_token,
        set_token=set_token_mock,
    )

    # setup two request mocks: the token refresh and the actual data request
    response.post("https://api.mollie.com/oauth2/tokens", "token_single")
    response.get("https://api.mollie.com/v2/organizations/me", "organization_current")

    organization = client.organizations.get('me')
    assert isinstance(organization, Organization), "Unexpected result from request."
    assert response.assert_all_requests_are_fired, "Not all expected requests have been performed."

    # verify handling of the new token
    set_token_mock.assert_called_once()
    args, kwargs = set_token_mock.call_args
    assert isinstance(args[0], dict), "set_token() did not receive a dictionary."



from mollie.api.objects.order import Order
from mollie.api.objects.order_line import OrderLine
from mollie.api.objects.payment import Payment
from mollie.api.objects.refund import Refund

from .utils import assert_list_object

PAYMENT_ID = 'tr_7UhSN1zuXS'
REFUND_ID = 're_4qqhO89gsT'
ORDER_ID = 'ord_kEn1PlbGa'


def test_get_refund(client, response):
    """Retrieve a specific refund of a payment."""
    response.get('https://api.mollie.com/v2/payments/%s/refunds/%s' % (PAYMENT_ID, REFUND_ID), 'refund_single')
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')
    response.get('https://api.mollie.com/v2/orders/{order_id}'.format(order_id=ORDER_ID), 'order_single')

    refund = client.payment_refunds.with_parent_id(PAYMENT_ID).get(REFUND_ID)
    assert isinstance(refund, Refund)
    # properties
    assert refund.resource == 'refund'
    assert refund.id == REFUND_ID
    assert refund.amount == {'currency': 'EUR', 'value': '5.95'}
    assert refund.settlement_amount == {'currency': 'EUR', 'value': '10.00'}
    assert refund.description == 'Required quantity not in stock, refunding one photo book.'
    assert refund.metadata == {'bookkeeping_id': 12345}
    assert refund.status == Refund.STATUS_PENDING
    assert_list_object(refund.lines, OrderLine)
    assert refund.payment_id == PAYMENT_ID
    assert refund.order_id == ORDER_ID
    assert refund.created_at == '2018-03-14T17:09:02.0Z'
    # properties from _links
    assert refund.payment is not None
    assert isinstance(refund.order, Order)
    # additional methods
    assert refund.is_queued() is False
    assert refund.is_pending() is True
    assert refund.is_processing() is False
    assert refund.is_refunded() is False


def test_refund_get_related_payment(client, response):
    """Verify the related payment of a refund."""
    response.get('https://api.mollie.com/v2/payments/%s/refunds/%s' % (PAYMENT_ID, REFUND_ID), 'refund_single')
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')

    refund = client.payment_refunds.with_parent_id(PAYMENT_ID).get(REFUND_ID)
    payment = refund.payment
    assert isinstance(payment, Payment)
    assert payment.id == PAYMENT_ID


def test_create_refund(client, response):
    """Create a payment refund of a payment."""
    response.post('https://api.mollie.com/v2/payments/%s/refunds' % PAYMENT_ID, 'refund_single')

    data = {
        'amount': {'value': '5.95', 'currency': 'EUR'}
    }
    refund = client.payment_refunds.with_parent_id(PAYMENT_ID).create(data)
    assert isinstance(refund, Refund)
    assert refund.id == REFUND_ID


def test_get_single_refund_on_payment_object(client, response):
    """Retrieve a payment refund of a payment."""
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')
    response.get('https://api.mollie.com/v2/payments/%s/refunds/%s' % (PAYMENT_ID, REFUND_ID), 'refund_single')

    payment = client.payments.get(PAYMENT_ID)
    refund = client.payment_refunds.on(payment).get(REFUND_ID)
    assert isinstance(refund, Refund)
    assert refund.id == REFUND_ID


def test_list_refunds_on_payment_object(client, response):
    """Retrieve a list of payment refunds of a payment."""
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')
    response.get('https://api.mollie.com/v2/payments/%s/refunds' % PAYMENT_ID, 'refunds_list')

    payment = client.payments.get(PAYMENT_ID)
    refunds = client.payment_refunds.on(payment).list()
    assert_list_object(refunds, Refund)


def test_cancel_refund(client, response):
    """Cancel a refund of a payment."""
    response.delete('https://api.mollie.com/v2/payments/%s/refunds/%s' % (PAYMENT_ID, REFUND_ID), 'empty')

    canceled_refund = client.payment_refunds.with_parent_id(PAYMENT_ID).delete(REFUND_ID)
    assert canceled_refund == {}



from ..error import IdentifierError
from ..objects.invoice import Invoice
from .base import Base


class Invoices(Base):
    RESOURCE_ID_PREFIX = 'inv_'

    def get_resource_object(self, result):
        return Invoice(result, self.client)

    def get(self, invoice_id, **params):
        if not invoice_id or not invoice_id.startswith(self.RESOURCE_ID_PREFIX):
            raise IdentifierError(
                "Invalid invoice ID: '{id}'. A invoice ID should start with '{prefix}'.".format(
                    id=invoice_id, prefix=self.RESOURCE_ID_PREFIX)
            )
        return super().get(invoice_id, **params)



from mollie.api.objects.list import List
from mollie.api.objects.method import Method


def test_list_iterator_behaviour(client, response):
    """Verify the behaviour of the List object in iterator circumstances."""
    response.get('https://api.mollie.com/v2/methods', 'methods_list')

    methods = client.methods.list()
    assert isinstance(methods, List)

    # walk the list using next()
    iterated = 0
    while True:
        try:
            item = next(methods)
            assert isinstance(item, Method)
            iterated += 1
        except StopIteration:
            # read all items from the iterator
            break

    assert iterated == methods.count

    # use for loop as iterator
    iterated = 0
    for item in methods:
        assert isinstance(item, Method)
        iterated += 1

    assert iterated == methods.count


def test_list_multiple_iterations(client, response):
    """Verify that we can iterate a list multiple times."""
    response.get('https://api.mollie.com/v2/methods', 'methods_list')

    methods = client.methods.list()
    # iterate once using next()
    items_first_run = 0
    while True:
        try:
            next(methods)
            items_first_run += 1
        except StopIteration:
            break
    assert items_first_run == methods.count

    # iterate again
    items_second_run = 0
    while True:
        try:
            next(methods)
            items_second_run += 1
        except StopIteration:
            break
    assert items_second_run == methods.count

    methods = client.methods.list()
    # iterate once using for loop
    items_third_run = 0
    for item in methods:
        items_third_run += 1
    assert items_third_run == methods.count
    # iterate again
    items_fourth_run = 0
    for item in methods:
        items_fourth_run += 1
    assert items_fourth_run == methods.count


def test_list_multiple_api_calls(client, response):
    """Verify that we can iterate over a paginated result set."""
    response.get('https://api.mollie.com/v2/customers?limit=5', 'customers_list_first')
    response.get('https://api.mollie.com/v2/customers?from=cst_8pknKQJzJa&limit=5', 'customers_list_second')
    response.get('https://api.mollie.com/v2/customers?from=cst_prs8JjDf57&limit=5', 'customers_list_third')
    response.get('https://api.mollie.com/v2/customers?from=cst_g328m9rhGe&limit=5', 'customers_list_fourth')
    response.get('https://api.mollie.com/v2/customers?from=cst_prs8JjDf57&limit=5', 'customers_list_third')
    response.get('https://api.mollie.com/v2/customers?from=cst_8pknKQJzJa&limit=5', 'customers_list_second')
    response.get('https://api.mollie.com/v2/customers?from=cst_HwBHgJgRAf&limit=5', 'customers_list_first')

    customers = client.customers.list(limit=5)
    assert customers.count == 5
    forward_first_id = next(customers).id
    customer_ids = [customer.id for customer in customers]
    while customers.has_next():
        customers = customers.get_next()
        for customer in customers:
            customer_ids.append(customer.id)

    assert len(customer_ids) == 17, 'Unexpected number of customers'
    assert len(set(customer_ids)) == 17, 'Unexpected number of unique customers'

    # now reverse the behaviour
    while customers.has_previous():
        customers = customers.get_previous()
    reverse_last_id = next(customers).id

    assert forward_first_id == reverse_last_id, \
        "Expected the first customer of the first page to be stable, but it wasn't"



from .base import Base


class Profile(Base):
    STATUS_UNVERIFIED = 'unverified'
    STATUS_VERIFIED = 'verified'
    STATUS_BLOCKED = 'blocked'

    @classmethod
    def get_resource_class(cls, client):
        from ..resources.profiles import Profiles
        return Profiles(client)

    @property
    def id(self):
        return self._get_property('id')

    @property
    def resource(self):
        return self._get_property('resource')

    @property
    def mode(self):
        return self._get_property('mode')

    @property
    def name(self):
        return self._get_property('name')

    @property
    def website(self):
        return self._get_property('website')

    @property
    def email(self):
        return self._get_property('email')

    @property
    def phone(self):
        return self._get_property('phone')

    @property
    def category_code(self):
        return self._get_property('categoryCode')

    @property
    def status(self):
        return self._get_property('status')

    @property
    def review(self):
        return self._get_property('review')

    @property
    def created_at(self):
        return self._get_property('createdAt')

    @property
    def chargebacks(self):
        return self.client.profile_chargebacks.on(self).list()

    @property
    def methods(self):
        return self.client.profile_methods.on(self).list()

    @property
    def payments(self):
        return self.client.profile_payments.on(self).list()

    @property
    def refunds(self):
        return self.client.profile_refunds.on(self).list()

    @property
    def checkout_preview_url(self):
        return self._get_link('checkoutPreviewUrl')

    # additional methods

    def is_unverified(self):
        return self._get_property('status') == self.STATUS_UNVERIFIED

    def is_verified(self):
        return self._get_property('status') == self.STATUS_VERIFIED

    def is_blocked(self):
        return self._get_property('status') == self.STATUS_BLOCKED



import json

import pytest

from mollie.api.error import APIDeprecationWarning, IdentifierError
from mollie.api.objects.settlement import Settlement
from mollie.api.resources.settlements import Settlements
from tests.utils import assert_list_object

SETTLEMENT_ID = 'stl_jDk30akdN'


def test_list_settlements(oauth_client, response):
    """Get a list of settlements."""
    response.get('https://api.mollie.com/v2/settlements', 'settlements_list')

    settlements = oauth_client.settlements.list()
    assert_list_object(settlements, Settlement)


def test_settlement_get(oauth_client, response):
    """Retrieve a single settlement method by ID."""
    response.get('https://api.mollie.com/v2/settlements/%s' % SETTLEMENT_ID, 'settlement_single')
    response.get('https://api.mollie.com/v2/settlements/%s/chargebacks' % SETTLEMENT_ID, 'chargebacks_list')
    response.get('https://api.mollie.com/v2/settlements/%s/payments' % SETTLEMENT_ID, 'settlement_payments_multiple')
    response.get('https://api.mollie.com/v2/settlements/%s/refunds' % SETTLEMENT_ID, 'refunds_list')

    settlement = oauth_client.settlements.get(SETTLEMENT_ID)
    chargebacks = oauth_client.settlement_chargebacks.with_parent_id(SETTLEMENT_ID).list()
    payments = oauth_client.settlement_payments.with_parent_id(SETTLEMENT_ID).list()
    refunds = oauth_client.settlement_refunds.with_parent_id(SETTLEMENT_ID).list()

    assert isinstance(settlement, Settlement)

    assert settlement.periods == json.loads(
        """{
            "2018": {
                "4": {
                    "revenue": [
                        {
                            "description": "iDEAL",
                            "method": "ideal",
                            "count": 6,
                            "amountNet": {
                                "currency": "EUR",
                                "value": "86.1000"
                            },
                            "amountVat": null,
                            "amountGross": {
                                "currency": "EUR",
                                "value": "86.1000"
                            }
                        },
                        {
                            "description": "Refunds iDEAL",
                            "method": "refund",
                            "count": 2,
                            "amountNet": {
                                "currency": "EUR",
                                "value": "-43.2000"
                            },
                            "amountVat": null,
                            "amountGross": {
                                "currency": "EUR",
                                "value": "43.2000"
                            }
                        }
                    ],
                    "costs": [
                        {
                            "description": "iDEAL",
                            "method": "ideal",
                            "count": 6,
                            "rate": {
                                "fixed": {
                                    "currency": "EUR",
                                    "value": "0.3500"
                                },
                                "percentage": null
                            },
                            "amountNet": {
                                "currency": "EUR",
                                "value": "2.1000"
                            },
                            "amountVat": {
                                "currency": "EUR",
                                "value": "0.4410"
                            },
                            "amountGross": {
                                "currency": "EUR",
                                "value": "2.5410"
                            }
                        },
                        {
                            "description": "Refunds iDEAL",
                            "method": "refund",
                            "count": 2,
                            "rate": {
                                "fixed": {
                                    "currency": "EUR",
                                    "value": "0.2500"
                                },
                                "percentage": null
                            },
                            "amountNet": {
                                "currency": "EUR",
                                "value": "0.5000"
                            },
                            "amountVat": {
                                "currency": "EUR",
                                "value": "0.1050"
                            },
                            "amountGross": {
                                "currency": "EUR",
                                "value": "0.6050"
                            }
                        }
                    ],
                    "invoiceId": "inv_FrvewDA3Pr"
                }
            }
        }"""
    )

    assert settlement.reference == '1234567.1804.03'
    assert settlement.created_at == '2018-04-06T06:00:01.0Z'
    assert settlement.settled_at == '2018-04-06T09:41:44.0Z'
    assert settlement.amount == {'currency': 'EUR', 'value': '39.75'}

    assert settlement.status == settlement.STATUS_OPEN
    assert settlement.is_open() is True
    assert settlement.is_canceled() is False
    assert settlement.is_failed() is False
    assert settlement.is_pending() is False

    assert settlement.chargebacks == chargebacks
    assert settlement.payments == payments
    assert settlement.refunds == refunds


def test_settlement_get_next(oauth_client, response):
    """Retrieve the details of the current settlement that has not yet been paid out."""
    response.get('https://api.mollie.com/v2/settlements/next', 'settlement_next')

    settlement = oauth_client.settlements.get('next')

    assert isinstance(settlement, Settlement)
    assert settlement.created_at == '2018-04-06T06:00:01.0Z'
    assert settlement.settled_at is None
    assert settlement.amount == {'currency': 'EUR', 'value': '39.75'}


def test_settlement_get_open(oauth_client, response):
    """Retrieve the details of the open balance of the organization."""
    response.get('https://api.mollie.com/v2/settlements/open', 'settlement_open')

    settlement = oauth_client.settlements.get('open')

    assert isinstance(settlement, Settlement)
    assert settlement.reference is None
    assert settlement.created_at == '2018-04-06T06:00:01.0Z'
    assert settlement.settled_at is None
    assert settlement.amount == {'currency': 'EUR', 'value': '39.75'}


@pytest.mark.parametrize(
    "input,expected",
    [
        (None, IdentifierError),
        ("foo", IdentifierError),
        ("next", None),  # Valid
        ("open", None),  # Valid
        (SETTLEMENT_ID, None),  # Valid
        ("1234567.1234.12", None)  # Valid
    ]
)
def test_validate_settlement_id(input, expected):
    """Ensure that we only accept correctly formatted settlement IDs."""
    if expected == IdentifierError:
        with pytest.raises(IdentifierError):
            Settlements.validate_settlement_id(input)
    else:
        assert Settlements.validate_settlement_id(input) is expected


def test_settlement_invoice_id_is_deprecated(oauth_client, response):
    response.get('https://api.mollie.com/v2/settlements/%s' % SETTLEMENT_ID, 'settlement_single')

    settlement = oauth_client.settlements.get(SETTLEMENT_ID)
    with pytest.warns(APIDeprecationWarning, match='Using Settlement Invoice ID is deprecated'):
        assert settlement.invoice_id == "inv_FrvewDA3Pr"



from ..error import IdentifierError
from ..objects.profile import Profile
from .base import Base


class Profiles(Base):
    RESOURCE_ID_PREFIX = 'pfl_'

    def get_resource_object(self, result):
        return Profile(result, self.client)

    def get(self, profile_id, **params):
        if not profile_id or \
                (not profile_id.startswith(self.RESOURCE_ID_PREFIX)
                 and not profile_id == 'me'):
            raise IdentifierError(
                "Invalid profile ID: '{id}'. A profile ID should start with '{prefix}' "
                "or it should be 'me'.".format(
                    id=profile_id,
                    prefix=self.RESOURCE_ID_PREFIX)
            )
        return super().get(profile_id, **params)



from mollie.api.objects.chargeback import Chargeback

from .utils import assert_list_object

PAYMENT_ID = 'tr_7UhSN1zuXS'
CHARGEBACK_ID = 'chb_n9z0tp'


def test_get_payment_chargebacks_by_payment_id(client, response):
    """Get chargebacks relevant to payment by payment id."""
    response.get('https://api.mollie.com/v2/payments/%s/chargebacks' % PAYMENT_ID, 'chargebacks_list')

    chargebacks = client.payment_chargebacks.with_parent_id(PAYMENT_ID).list()
    assert_list_object(chargebacks, Chargeback)


def test_get_single_payment_chargeback(client, response):
    """Get a single chargeback relevant to payment by payment id."""
    response.get('https://api.mollie.com/v2/payments/%s/chargebacks/%s' % (PAYMENT_ID, CHARGEBACK_ID),
                 'chargeback_single')

    chargeback = client.payment_chargebacks.with_parent_id(PAYMENT_ID).get(CHARGEBACK_ID)
    assert isinstance(chargeback, Chargeback)
    assert chargeback.id == CHARGEBACK_ID
    assert chargeback.amount == {'currency': 'USD', 'value': '43.38'}
    assert chargeback.settlement_amount == {'currency': 'EUR', 'value': '-35.07'}
    assert chargeback.created_at == '2018-03-14T17:00:52.0Z'
    assert chargeback.reversed_at == '2018-03-14T17:00:55.0Z'
    assert chargeback.payment_id == PAYMENT_ID


def test_list_payment_chargebacks_by_payment_object(client, response):
    """Get a list of chargebacks relevant to payment object."""
    response.get('https://api.mollie.com/v2/payments/%s/chargebacks' % PAYMENT_ID, 'chargebacks_list')
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')

    payment = client.payments.get(PAYMENT_ID)
    chargebacks = client.payment_chargebacks.on(payment).list()
    assert_list_object(chargebacks, Chargeback)


def test_get_single_payment_chargeback_by_payment_object(client, response):
    """Get a single chargeback relevant to payment object."""
    response.get('https://api.mollie.com/v2/payments/%s/chargebacks/%s' % (PAYMENT_ID, CHARGEBACK_ID),
                 'chargeback_single')
    response.get('https://api.mollie.com/v2/payments/%s' % PAYMENT_ID, 'payment_single')

    payment = client.payments.get(PAYMENT_ID)
    chargeback = client.payment_chargebacks.on(payment).get(CHARGEBACK_ID)
    assert isinstance(chargeback, Chargeback)
    assert chargeback.payment_id == PAYMENT_ID



from mollie.api.objects.customer import Customer
from mollie.api.objects.mandate import Mandate

from .utils import assert_list_object

CUSTOMER_ID = 'cst_8wmqcHMN4U'
MANDATE_ID = 'mdt_h3gAaD5zP'


def test_list_customer_mandates(client, response):
    """Retrieve a list of mandates."""
    response.get('https://api.mollie.com/v2/customers/%s/mandates' % CUSTOMER_ID, 'customer_mandates_list')
    mandates = client.customer_mandates.with_parent_id(CUSTOMER_ID).list()
    assert_list_object(mandates, Mandate)


def test_get_customer_mandate_by_id(client, response):
    """Retrieve a single mandate by ID."""
    response.get(
        'https://api.mollie.com/v2/customers/%s/mandates/%s' % (CUSTOMER_ID, MANDATE_ID),
        'customer_mandate_single',
    )
    response.get('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_new')

    mandate = client.customer_mandates.with_parent_id(CUSTOMER_ID).get(MANDATE_ID)
    assert isinstance(mandate, Mandate)
    assert mandate.id == MANDATE_ID
    assert mandate.resource == 'mandate'
    assert mandate.status == 'valid'
    assert mandate.method == 'directdebit'
    assert mandate.details == {
        'consumerName': 'John Doe',
        'consumerAccount': 'NL55INGB0000000000',
        'consumerBic': 'INGBNL2A',
    }
    assert mandate.mandate_reference == 'YOUR-COMPANY-MD1380'
    assert mandate.signature_date == '2018-05-07'
    assert mandate.created_at == '2018-05-07T10:49:08+00:00'
    assert mandate.is_pending() is False
    assert mandate.is_valid() is True
    assert mandate.is_invalid() is False
    assert mandate.customer is not None


def test_get_customer_mandate_by_customer(client, response):
    """Retrieve a customer by customer object."""
    response.get('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_new')
    response.get('https://api.mollie.com/v2/customers/%s/mandates' % CUSTOMER_ID, 'customer_mandates_list')
    response.get(
        'https://api.mollie.com/v2/customers/%s/mandates/%s' % (CUSTOMER_ID, MANDATE_ID),
        'customer_mandate_single',
    )
    customer = client.customers.get(CUSTOMER_ID)

    mandates = client.customer_mandates.on(customer).list()
    assert_list_object(mandates, Mandate)

    mandate = client.customer_mandates.on(customer).get(MANDATE_ID)
    assert isinstance(mandate, Mandate)
    assert mandate.id == MANDATE_ID


def test_customer_mandate_get_related_customer(client, response):
    """Retrieve a related customer object from a mandate."""
    response.get(
        'https://api.mollie.com/v2/customers/%s/mandates/%s' % (CUSTOMER_ID, MANDATE_ID),
        'customer_mandate_single',
    )
    response.get('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_new')

    mandate = client.customer_mandates.with_parent_id(CUSTOMER_ID).get(MANDATE_ID)
    assert isinstance(mandate.customer, Customer)
    assert mandate.customer.id == CUSTOMER_ID


def test_customer_mandates_create_mandate(client, response):
    """Create a new customer mandate."""
    response.post('https://api.mollie.com/v2/customers/%s/mandates' % CUSTOMER_ID, 'customer_mandate_single')

    data = {
        'method': 'directdebit',
        'consumerName': 'John Doe',
        'consumerAccount': 'NL55INGB0000000000',
        'consumerBic': 'INGBNL2A',
        'signatureDate': '2018-05-07',
        'mandateReference': 'YOUR-COMPANY-MD1380',
    }
    mandate = client.customer_mandates.with_parent_id(CUSTOMER_ID).create(data=data)
    assert isinstance(mandate, Mandate)
    assert mandate.id == MANDATE_ID


def test_update_customer_mandate(client, response):
    response.patch(
        'https://api.mollie.com/v2/customers/%s/mandates/%s' % (CUSTOMER_ID, MANDATE_ID),
        'customer_mandate_updated'
    )

    data = {
        'method': 'directdebit',
        'consumerName': 'John Doe',
        'consumerAccount': 'NL09ASNB0000000000',
        'consumerBic': 'ASNBNL21',
        'signatureDate': '2018-05-07',
        'mandateReference': 'OTHER-COMPANY-12345',
    }
    mandate = client.customer_mandates.with_parent_id(CUSTOMER_ID).update(MANDATE_ID, data=data)
    assert isinstance(mandate, Mandate)
    assert mandate.id == MANDATE_ID


def test_revoke_customer_mandate(client, response):
    response.delete(
        'https://api.mollie.com/v2/customers/%s/mandates/%s' % (CUSTOMER_ID, MANDATE_ID),
        'empty',
        204
    )

    resp = client.customer_mandates.with_parent_id(CUSTOMER_ID).delete(MANDATE_ID)
    assert resp == {}



from .refunds import Refunds


class SettlementRefunds(Refunds):
    settlement_id = None

    def get_resource_name(self):
        return 'settlements/{id}/refunds'.format(id=self.settlement_id)

    def with_parent_id(self, settlement_id):
        self.settlement_id = settlement_id
        return self

    def on(self, settlement):
        return self.with_parent_id(settlement.id)



from mollie.api.objects.order import Order
from mollie.api.objects.payment import Payment
from mollie.api.objects.refund import Refund
from mollie.api.objects.shipment import Shipment

from .utils import assert_list_object

ORDER_ID = 'ord_kEn1PlbGa'


def test_get_order(client, response):
    """Retrieve a single order by order ID."""
    response.get('https://api.mollie.com/v2/orders/{order_id}'.format(order_id=ORDER_ID), 'order_single')
    response.get('https://api.mollie.com/v2/orders/{order_id}/shipments'.format(order_id=ORDER_ID), 'shipments_list')
    response.get('https://api.mollie.com/v2/orders/{order_id}/refunds'.format(order_id=ORDER_ID), 'order_refunds_list')

    order = client.orders.get(ORDER_ID)
    assert isinstance(order, Order)
    assert order.id == 'ord_kEn1PlbGa'
    assert order.resource == 'order'
    assert order.profile_id == 'pfl_URR55HPMGx'
    assert order.method == 'ideal'
    assert order.mode == 'live'
    assert order.amount == {'value': '1027.99', 'currency': 'EUR'}
    assert order.amount_captured is None
    assert order.amount_refunded is None
    assert order.status == 'created'
    assert order.is_cancelable is True
    assert order.billing_address == {
        "streetAndNumber": "Keizersgracht 313",
        "city": "Amsterdam",
        "region": "Noord-Holland",
        "postalCode": "1234AB",
        "country": "NL",
        "title": "Dhr.",
        "givenName": "Piet",
        "familyName": "Mondriaan",
        "email": "piet@mondriaan.com",
        "phone": "+31309202070"
    }
    assert order.consumer_date_of_birth == '1958-01-31'
    assert order.order_number == '1337'
    assert order.shipping_address == {
        'streetAndNumber': 'Keizersgracht 313',
        'streetAdditional': '4th floor',
        'city': 'Haarlem',
        'region': 'Noord-Holland',
        'postalCode': '5678AB',
        'country': 'NL',
        'title': 'Mr.',
        'givenName': 'Chuck',
        'familyName': 'Norris',
        'email': 'norris@chucknorrisfacts.net'
    }
    assert order.locale == 'nl_NL'
    assert order.metadata == {'description': 'Lego cars', 'order_id': '1337'}
    assert order.redirect_url == 'https://example.org/redirect'
    assert order.webhook_url == 'https://example.org/webhook'
    assert order.created_at == '2018-08-02T09:29:56+00:00'
    assert order.expires_at == '2018-08-30T09:29:56+00:00'
    assert order.expired_at is None
    assert order.paid_at is None
    assert order.authorized_at is None
    assert order.canceled_at is None
    assert order.completed_at is None
    assert order.checkout_url == 'https://www.mollie.com/payscreen/order/checkout/kEn1PlbGa'
    assert_list_object(order.shipments, Shipment)
    assert_list_object(order.refunds, Refund)

    assert order.is_created() is True
    assert order.is_paid() is False
    assert order.is_authorized() is False
    assert order.is_shipping() is False
    assert order.is_completed() is False
    assert order.is_expired() is False


def test_list_orders(client, response):
    """Retrieve a list of existing orders."""
    response.get('https://api.mollie.com/v2/orders', 'orders_list')

    orders = client.orders.list()
    assert_list_object(orders, Order)


def test_create_order_refund(client, response):
    """Create an order refund of an order."""
    response.get('https://api.mollie.com/v2/orders/{order_id}'.format(order_id=ORDER_ID), 'order_single')
    response.post('https://api.mollie.com/v2/orders/{order_id}/refunds'.format(order_id=ORDER_ID), 'refund_single')

    data = {
        "lines": [
            {
                "id": "odl_dgtxyl",
                "quantity": 1,
            }
        ],
        "description": "Required quantity not in stock, refunding one photo book.",
        "metadata": {
            "bookkeeping_id": 12345,
        }
    }

    order = client.orders.get(ORDER_ID)
    refund = order.create_refund(data)
    assert isinstance(refund, Refund)
    assert refund.status == Refund.STATUS_PENDING
    assert refund.description == "Required quantity not in stock, refunding one photo book."
    assert refund.metadata == {"bookkeeping_id": 12345}


def test_create_order(client, response):
    """Create an order."""
    response.post('https://api.mollie.com/v2/orders', 'order_single')

    data = {
        'amount': {
            'value': '1027.99',
            'currency': 'EUR'
        },
        'billingAddress': {
            'streetAndNumber': 'Keizersgracht 313',
            'city': 'Amsterdam',
            'region': 'Noord-Holland',
            'postalCode': '1234AB',
            'country': 'NL',
            'givenName': 'Piet',
            'familyName': 'Mondriaan',
            'email': 'piet@mondriaan.com',
        },
        'shippingAddress': {
            'streetAndNumber': 'Prinsengracht 313',
            'city': 'Haarlem',
            'region': 'Noord-Holland',
            'postalCode': '5678AB',
            'country': 'NL',
            'givenName': 'Chuck',
            'familyName': 'Norris',
            'email': 'norris@chucknorrisfacts.net'
        },
        'metadata': {
            'order_id': '1337',
            'description': 'Lego cars'
        },
        'consumerDateOfBirth': '1958-01-31',
        'locale': 'nl_NL',
        'orderNumber': '1337',
        'redirectUrl': 'https://example.org/redirect',
        'webhookUrl': 'https://example.org/webhook',
        'method': 'klarnapaylater',
        'lines': [
            {
                'type': 'physical',
                'sku': '5702016116977',
                'name': 'LEGO 42083 Bugatti Chiron',
                'productUrl': 'https://shop.lego.com/nl-NL/Bugatti-Chiron-42083',
                'imageUrl': 'https://sh-s7-live-s.legocdn.com/is/image//LEGO/42083_alt1?$main$',
                'quantity': 1,
                'vatRate': '21.00',
                'unitPrice': {
                    'currency': 'EUR',
                    'value': '399.00'
                },
                'totalAmount': {
                    'currency': 'EUR',
                    'value': '698.00'
                },
                'discountAmount': {
                    'currency': 'EUR',
                    'value': '100.00'
                },
                'vatAmount': {
                    'currency': 'EUR',
                    'value': '121.14'
                }
            },

        ]
    }
    order = client.orders.create(data)
    assert isinstance(order, Order)
    assert order.id == ORDER_ID


def test_update_order(client, response):
    """Update an existing order."""
    response.patch('https://api.mollie.com/v2/orders/{order_id}'.format(order_id=ORDER_ID), 'order_updated')

    data = {
        'billingAddress': {
            'streetAndNumber': 'Keizersgracht 313',
            'city': 'Amsterdam',
            'region': 'Noord-Holland',
            'postalCode': '1234AB',
            'country': 'NL',
            'title': 'Dhr',
            'givenName': 'Piet',
            'familyName': 'Mondriaan',
            'email': 'piet@mondriaan.com',
            'phone': '+31208202070'
        }
    }
    updated_order = client.orders.update(ORDER_ID, data)
    assert isinstance(updated_order, Order)
    assert updated_order.billing_address['givenName'] == 'Piet'


def test_cancel_order(client, response):
    """Cancel an existing order."""
    response.delete('https://api.mollie.com/v2/orders/{order_id}'.format(order_id=ORDER_ID), 'order_canceled', 200)

    canceled_order = client.orders.delete(ORDER_ID)
    assert isinstance(canceled_order, Order)
    assert canceled_order.is_canceled() is True
    assert canceled_order.is_cancelable is False


def test_cancel_order_lines(client, response):
    """Cancel a line of an order."""
    response.get('https://api.mollie.com/v2/orders/{order_id}'.format(order_id=ORDER_ID), 'order_single')
    response.delete('https://api.mollie.com/v2/orders/{order_id}/lines'.format(order_id=ORDER_ID), 'empty', 204)

    order = client.orders.get(ORDER_ID)
    line = next(order.lines)
    data = {
        'lines': [
            {
                'id': line.id,
                'quantity': line.quantity
            }
        ]
    }
    canceled = order.cancel_lines(data)
    assert canceled == {}


def test_create_order_payment(client, response):
    """Create a payment for an order."""
    response.get('https://api.mollie.com/v2/orders/{order_id}'.format(order_id=ORDER_ID), 'order_single')
    response.post('https://api.mollie.com/v2/orders/{order_id}/payments'.format(order_id=ORDER_ID), 'payment_single')

    order = client.orders.get(ORDER_ID)
    data = {'method': 'ideal'}
    payment = order.create_payment(data)
    assert isinstance(payment, Payment)
    assert payment.order_id == ORDER_ID



from mollie.api.objects.invoice import Invoice

from .utils import assert_list_object

INVOICE_ID = 'inv_xBEbP9rvAq'


def test_list_invoices(oauth_client, response):
    """Retrieve a list of existing invoices."""
    response.get('https://api.mollie.com/v2/invoices', 'invoices_list')

    invoices = oauth_client.invoices.list()
    assert_list_object(invoices, Invoice)


def test_get_invoice(oauth_client, response):
    """Retrieve a single invoice."""
    response.get('https://api.mollie.com/v2/invoices/%s' % INVOICE_ID, 'invoice_single')

    invoice = oauth_client.invoices.get(INVOICE_ID)
    assert isinstance(invoice, Invoice)
    assert invoice.id == INVOICE_ID
    assert invoice.resource == 'invoice'
    assert invoice.id == "inv_xBEbP9rvAq"
    assert invoice.reference == "2016.10000"
    assert invoice.vat_number == "NL001234567B01"
    assert invoice.status == "open"
    assert invoice.issued_at == "2016-08-31"
    assert invoice.due_at == "2016-09-14"
    assert invoice.net_amount == {"value": "45.00", "currency": "EUR"}
    assert invoice.vat_amount == {"value": "9.45", "currency": "EUR"}
    assert invoice.gross_amount == {"value": "54.45", "currency": "EUR"}
    assert invoice.lines == [
        {
            "period": "2016-09",
            "description": "iDEAL transactiekosten",
            "count": 100,
            "vatPercentage": 21,
            "amount": {
                "value": "45.00",
                "currency": "EUR"
            }
        }
    ]
    url = 'https://www.mollie.com/merchant/download/invoice/xBEbP9rvAq/2ab44d60b35b1d06090bba955fa2c602'
    assert invoice.pdf == url



from .base import Base


class Permission(Base):
    @classmethod
    def get_resource_class(cls, client):
        from ..resources.permissions import Permissions
        return Permissions(client)

    @property
    def id(self):
        return self._get_property('id')

    @property
    def resource(self):
        return self._get_property('resource')

    @property
    def description(self):
        return self._get_property('description')

    @property
    def granted(self):
        return self._get_property('granted')



from .base import Base


class Onboarding(Base):
    @classmethod
    def get_resource_class(cls, client):
        from ..resources.onboarding import Onboarding as OnboardingResource
        return OnboardingResource(client)

    STATUS_NEEDS_DATA = 'needs-data'
    STATUS_IN_REVIEW = 'in-review'  # Waiting for a valid mandate.
    STATUS_COMPLETED = 'completed'

    @property
    def resource(self):
        return self._get_property('resource')

    @property
    def name(self):
        return self._get_property('name')

    @property
    def signed_up_at(self):
        return self._get_property('signedUpAt')

    @property
    def status(self):
        return self._get_property('status')

    @property
    def can_receive_payments(self):
        return self._get_property('canReceivePayments')

    @property
    def can_receive_settlements(self):
        return self._get_property('canReceiveSettlements')

    @property
    def organization(self):
        """Retrieve organization for an onboarding."""
        from .organization import Organization
        url = self._get_link('organization')

        if url:
            resp = self.client.organizations.perform_api_call(self.client.organizations.REST_READ, url)
            return Organization(resp)

    def is_needs_data(self):
        return self.status == self.STATUS_NEEDS_DATA

    def is_in_review(self):
        return self.status == self.STATUS_IN_REVIEW

    def is_completed(self):
        return self.status == self.STATUS_COMPLETED



from .methods import Methods


class ProfileMethods(Methods):
    profile_id = None
    method_id = None

    def get_resource_name(self):
        return 'profiles/{id}/methods/{method}'.format(id=self.profile_id, method=self.method_id)

    def list(self, **params):
        return self.client.methods.list(profileId=self.profile_id)

    def delete(self, *args, **kwargs):
        path = self.get_resource_name()
        return self.perform_api_call(self.REST_DELETE, path, None)

    def with_parent_id(self, profile_id, method=None):
        self.method_id = method
        self.profile_id = profile_id
        return self

    def on(self, profile, method=None):
        return self.with_parent_id(profile.id, method)



from .refunds import Refunds


class PaymentRefunds(Refunds):
    payment_id = None

    def get_resource_name(self):
        return 'payments/{id}/refunds'.format(id=self.payment_id)

    def with_parent_id(self, payment_id):
        self.payment_id = payment_id
        return self

    def on(self, payment):
        return self.with_parent_id(payment.id)



from mollie.api.objects.order import Order
from mollie.api.objects.order_line import OrderLine
from mollie.api.objects.shipment import Shipment

from .utils import assert_list_object

ORDER_ID = 'ord_kEn1PlbGa'
SHIPMENT_ID = 'shp_3wmsgCJN4U'


def test_get_shipment(client, response):
    """Retrieve a single shipment by a shipment's ID."""
    response.get('https://api.mollie.com/v2/orders/{order_id}'.format(order_id=ORDER_ID), 'order_single')
    response.get('https://api.mollie.com/v2/orders/{order_id}/shipments/{shipment_id}'.format(
        order_id=ORDER_ID, shipment_id=SHIPMENT_ID), 'shipment_single')

    order = client.orders.get(ORDER_ID)
    shipment = order.get_shipment(SHIPMENT_ID)
    assert isinstance(shipment, Shipment)
    assert shipment.resource == 'shipment'
    assert shipment.id == SHIPMENT_ID
    assert shipment.order_id == ORDER_ID
    assert shipment.created_at == '2018-08-09T14:33:54+00:00'
    assert shipment.tracking == {
        'carrier': 'PostNL',
        'code': '3SKABA000000000',
        'url': 'http://postnl.nl/tracktrace/?B=3SKABA000000000&P=1016EE&D=NL&T=C'
    }
    assert shipment.has_tracking() is True
    assert shipment.has_tracking_url() is True
    assert shipment.tracking_url == 'http://postnl.nl/tracktrace/?B=3SKABA000000000&P=1016EE&D=NL&T=C'
    assert_list_object(shipment.lines, OrderLine)
    assert isinstance(shipment.order, Order)


def test_create_shipment(client, response):
    """Create a shipment of an order object"""
    response.get('https://api.mollie.com/v2/orders/{order_id}'.format(order_id=ORDER_ID), 'order_single')
    response.post('https://api.mollie.com/v2/orders/ord_kEn1PlbGa/shipments', 'shipment_single')

    data = {
        'lines': [
            {
                'id': 'odl_dgtxyl',
                'quantity': 1
            },
            {
                'id': 'odl_dgtxyb'
            }
        ],
        'tracking': {
            'carrier': 'PostNL',
            'code': '3SKABA000000000',
            'url': 'http://postnl.nl/tracktrace/?B=3SKABA000000000&P=1016EE&D=NL&T=C'
        },
    }
    order = client.orders.get(ORDER_ID)
    new_shipment = order.create_shipment(data)
    assert isinstance(new_shipment, Shipment)


def test_update_shipment(client, response):
    """Update the tracking information of a shipment"""
    response.get('https://api.mollie.com/v2/orders/{order_id}'.format(order_id=ORDER_ID), 'order_single')
    response.patch('https://api.mollie.com/v2/orders/{order_id}/shipments/{shipment_id}'.format(
        order_id=ORDER_ID, shipment_id=SHIPMENT_ID), 'shipment_single')

    order = client.orders.get(ORDER_ID)
    data = {
         'tracking': {
             'carrier': 'PostNL',
             'code': '3SKABA000000000',
             'url': 'http://postnl.nl/tracktrace/?B=3SKABA000000000&P=1016EE&D=NL&T=C'
         },
     }
    updated_shipment = order.update_shipment(SHIPMENT_ID, data)
    assert isinstance(updated_shipment, Shipment)
    assert updated_shipment.id == SHIPMENT_ID



from mollie.api.objects.payment import Payment

from .utils import assert_list_object

CUSTOMER_ID = 'cst_8wmqcHMN4U'


def test_list_customer_payments(client, response):
    """Retrieve a list of payments related to a customer id."""
    response.get('https://api.mollie.com/v2/customers/%s/payments' % CUSTOMER_ID, 'customer_payments_multiple')

    payments = client.customer_payments.with_parent_id(CUSTOMER_ID).list()
    assert_list_object(payments, Payment)


def test_list_customer_payments_by_object(client, response):
    response.get('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_single')
    response.get('https://api.mollie.com/v2/customers/%s/payments' % CUSTOMER_ID, 'customer_payments_multiple')

    customer = client.customers.get(CUSTOMER_ID)
    payments = client.customer_payments.on(customer).list()
    assert_list_object(payments, Payment)


def test_create_customer_payment(client, response):
    """Create a customer payment."""
    response.post('https://api.mollie.com/v2/customers/%s/payments' % CUSTOMER_ID, 'payment_single')

    payment = client.customer_payments.with_parent_id(CUSTOMER_ID).create(
        {
            'amount': {'currency': 'EUR', 'value': '10.00'},
            'description': 'Order #12345',
            'redirectUrl': 'https://webshop.example.org/order/12345/',
            'webhookUrl': 'https://webshop.example.org/payments/webhook/',
            'method': 'ideal',
        })
    assert isinstance(payment, Payment)
    assert payment.customer_id == CUSTOMER_ID



from mollie.api.objects.customer import Customer
from mollie.api.objects.mandate import Mandate
from mollie.api.objects.payment import Payment
from mollie.api.objects.subscription import Subscription

from .utils import assert_list_object

CUSTOMER_ID = 'cst_8wmqcHMN4U'


def test_create_customer(client, response):
    """Create a new customer."""
    response.post('https://api.mollie.com/v2/customers', 'customer_new')

    customer = client.customers.create({
        'name': 'Customer A',
        'email': 'customer@example.org',
        'locale': 'nl_NL',
    })
    assert isinstance(customer, Customer)
    assert customer.id == CUSTOMER_ID


def test_update_customer(client, response):
    """Update an existing customer."""
    response.patch('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_updated')

    updated_customer = client.customers.update(CUSTOMER_ID, {
        'name': 'Updated Customer A',
        'email': 'updated-customer@example.org',
    })
    assert isinstance(updated_customer, Customer)
    assert updated_customer.name == 'Updated Customer A'
    assert updated_customer.email == 'updated-customer@example.org'


def test_delete_customer(client, response):
    """Delete a customer."""
    response.delete('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'empty')

    deleted_customer = client.customers.delete('cst_8wmqcHMN4U')
    assert deleted_customer == {}


def test_list_customers(client, response):
    """Retrieve a list of existing customers."""
    response.get('https://api.mollie.com/v2/customers', 'customers_list')

    customers = client.customers.list()
    assert_list_object(customers, Customer)


def test_get_customer(client, response):
    """Retrieve a single customer."""
    response.get('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_new')
    response.get('https://api.mollie.com/v2/customers/%s/subscriptions' % CUSTOMER_ID, 'subscriptions_customer_list')
    response.get('https://api.mollie.com/v2/customers/%s/mandates' % CUSTOMER_ID, 'customer_mandates_list')
    response.get('https://api.mollie.com/v2/customers/%s/payments' % CUSTOMER_ID, 'customer_payments_multiple')

    customer = client.customers.get(CUSTOMER_ID)
    assert isinstance(customer, Customer)
    assert customer.id == CUSTOMER_ID
    assert customer.name == 'Customer A'
    assert customer.email == 'customer@example.org'
    assert customer.locale == 'nl_NL'
    assert customer.metadata == {'orderId': '12345'}
    assert customer.mode == 'test'
    assert customer.resource == 'customer'
    assert customer.created_at == '2018-04-06T13:10:19.0Z'
    assert customer.subscriptions is not None
    assert customer.mandates is not None
    assert customer.payments is not None


def test_customer_get_related_mandates(client, response):
    """Retrieve related mandates for a customer."""
    response.get('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_updated')
    response.get('https://api.mollie.com/v2/customers/%s/mandates' % CUSTOMER_ID, 'customer_mandates_list')

    customer = client.customers.get(CUSTOMER_ID)
    mandates = customer.mandates
    assert_list_object(mandates, Mandate)


def test_customer_get_related_subscriptions(client, response):
    """Retrieve related subscriptions for a customer."""
    response.get('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_single')
    response.get('https://api.mollie.com/v2/customers/%s/subscriptions' % CUSTOMER_ID,
                 'subscriptions_customer_list')

    customer = client.customers.get(CUSTOMER_ID)
    subscriptions = customer.subscriptions
    assert_list_object(subscriptions, Subscription)


def test_customer_get_related_payments(client, response):
    """Retrieve related payments for a customer."""
    response.get('https://api.mollie.com/v2/customers/%s' % CUSTOMER_ID, 'customer_new')
    response.get('https://api.mollie.com/v2/customers/%s/payments' % CUSTOMER_ID, 'customer_payments_multiple')

    customer = client.customers.get(CUSTOMER_ID)
    payments = customer.payments
    assert_list_object(payments, Payment)



from mollie.api.objects.chargeback import Chargeback

from .utils import assert_list_object

PAYMENT_ID = 'tr_7UhSN1zuXS'
SETTLEMENT_ID = 'stl_jDk30akdN'
CHARGEBACK_ID = 'chb_n9z0tp'


def test_get_settlement_chargebacks_by_chargeback_id(oauth_client, response):
    """Get chargebacks relevant to settlement by settlement id."""
    response.get('https://api.mollie.com/v2/settlements/%s/chargebacks' % SETTLEMENT_ID, 'chargebacks_list')

    chargebacks = oauth_client.settlement_chargebacks.with_parent_id(SETTLEMENT_ID).list()
    assert_list_object(chargebacks, Chargeback)


def test_list_settlement_chargebacks_by_settlement_object(oauth_client, response):
    """Get a list of chargebacks relevant to settlement object."""
    response.get('https://api.mollie.com/v2/settlements/%s/chargebacks' % SETTLEMENT_ID, 'chargebacks_list')
    response.get('https://api.mollie.com/v2/settlements/%s' % SETTLEMENT_ID, 'settlement_single')

    settlement = oauth_client.settlements.get(SETTLEMENT_ID)
    chargebacks = oauth_client.settlement_chargebacks.on(settlement).list()
    assert_list_object(chargebacks, Chargeback)



from mollie.api.objects.issuer import Issuer

from .utils import assert_list_object


def test_get_issuers(client, response):
    """Get all the iDeal issuers via the include querystring parameter."""
    response.get('https://api.mollie.com/v2/methods/ideal?include=issuers', 'method_get_ideal_with_includes')

    issuers = client.methods.get('ideal', include='issuers').issuers
    assert_list_object(issuers, Issuer)

    # check a single retrieved issuer
    issuer = next(issuers)
    assert issuer.image_svg == 'https://www.mollie.com/external/icons/ideal-issuers/ABNANL2A.svg'
    assert issuer.image_size1x == 'https://www.mollie.com/images/checkout/v2/ideal-issuer-icons/ABNANL2A.png'
    assert issuer.image_size2x == 'https://www.mollie.com/images/checkout/v2/ideal-issuer-icons/ABNANL2A%402x.png'
    assert issuer.name == 'ABN AMRO'
    assert issuer.resource == 'issuer'
    assert issuer.id == 'ideal_ABNANL2A'



from ..error import IdentifierError
from ..objects.organization import Organization
from .base import Base


class Organizations(Base):
    RESOURCE_ID_PREFIX = 'org_'

    def get_resource_object(self, result):
        return Organization(result, self.client)

    def get(self, organization_id, **params):
        if not organization_id or \
                (not organization_id.startswith(self.RESOURCE_ID_PREFIX)
                 and not organization_id == 'me'):
            raise IdentifierError(
                "Invalid organization ID: '{id}'. A organization ID should start with '{prefix}' "
                "or it should be 'me'.".format(
                    id=organization_id,
                    prefix=self.RESOURCE_ID_PREFIX)
            )
        return super().get(organization_id, **params)



from .base import Base


class Customer(Base):
    @classmethod
    def get_resource_class(cls, client):
        from ..resources.customers import Customers
        return Customers(client)

    @property
    def id(self):
        return self._get_property('id')

    @property
    def name(self):
        return self._get_property('name')

    @property
    def email(self):
        return self._get_property('email')

    @property
    def locale(self):
        return self._get_property('locale')

    @property
    def metadata(self):
        return self._get_property('metadata')

    @property
    def mode(self):
        return self._get_property('mode')

    @property
    def resource(self):
        return self._get_property('resource')

    @property
    def created_at(self):
        return self._get_property('createdAt')

    @property
    def subscriptions(self):
        """Return the subscription list for the customer."""
        return self.client.customer_subscriptions.on(self).list()

    @property
    def mandates(self):
        """Return the mandate list for the customer."""
        return self.client.customer_mandates.on(self).list()

    @property
    def payments(self):
        """Return the payment list for the customer."""
        return self.client.customer_payments.on(self).list()
